#!/usr/bin/python
from albatross.cgiapp import Request
from albatross import SimpleSessionApp

node_num = 0


class Node:

    def __init__(self, name, children = None):
        self.name = name
        if children is not None:
            self.children = children
        self.selected = 0
        global node_num
        self.node_num = node_num
        node_num = node_num + 1

    def albatross_alias(self):
        return 'node%d' % self.node_num


class TreePage:

    def page_enter(self, ctx):
        ctx.locals.tree \
            = Node('a', [Node('a', [Node('a'),
                                    Node('b')]),
                         Node('b', [Node('a', [Node('a', [Node('a'),
                                                          Node('b')])]),
                                    Node('b'),
                                    Node('c', [Node('a'),
                                               Node('b')])])])
        ctx.add_session_vars('tree')

    def page_display(self, ctx):
        ctx.run_template('tree.html')


app = SimpleSessionApp(base_url='tree.py',
                       template_path='.',
                       start_page='start',
                       secret='-=-secret-=-',
                       session_appid='tree')

app.register_page('start', TreePage())


if __name__ == '__main__':
    app.run(Request())



#!/usr/bin/python
from albatross import SimpleSessionApp
from albatross.cgiapp import Request


class ListPage:

    def page_display(self, ctx):
        ctx.locals.data = range(100)
        ctx.run_template('paginate.html')


app = SimpleSessionApp(base_url='paginate.py',
                       template_path='.',
                       start_page='start',
                       secret='-=-secret-=-',
                       session_appid='paginate')
app.register_page('start', ListPage())


if __name__ == '__main__':
    app.run(Request())



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# Minimum functionality which is required for the template interpreter
# to run
import os
import sys
import stat
import re
import cPickle

try:
    import zlib
    have_zlib = 1
except ImportError:
    have_zlib = 0
import base64
import md5

from albatross.template import Template
from albatross.common import *

# old cPickle used string exceptions, and therefore couldn't subclass a common
# exception. New cPickle has a common cPickle.PickleError, but will also raise
# some other random exceptions - bah - these are the ones I've seen:
pickle_errors = (cPickle.PicklingError, 
                 cPickle.UnpicklingError,
                 TypeError,
                 EOFError)

# Templates execute within an execution context.  The context is
# required to handle the following:
#
# - HTML output
# - Macro and function arguments
# - Expression evaluation
# - Iterators
# - Global resources; templates, lookups, macros
#
# Applications which are session based will extend the execution
# context to prime it with values retreived from the session.

# ------------------------------------------------------------------
# Template interpreter resources - for all templates in application
# ------------------------------------------------------------------

class ResourceMixin:

    '''Maintains a global registry of tags, macros, and lookups.
    '''
    def __init__(self):
        self.__macros = {}
        self.__lookups = {}
        self.__tags = {}

    def get_macro(self, name):
        return self.__macros.get(name)

    def register_macro(self, name, macro):
        self.__macros[name] = macro

    def get_lookup(self, name):
        return self.__lookups.get(name)

    def register_lookup(self, name, lookup):
        self.__lookups[name] = lookup

    def get_tagclass(self, name):
        return self.__tags.get(name)

    def register_tagclasses(self, *tags):
        for tag in tags:
            self.__tags[tag.name] = tag

# ------------------------------------------------------------------
# Template interpreter "stack" - used to execute a single template
# ------------------------------------------------------------------

class ExecuteMixin:

    '''Manages a template execution context
    '''
    def __init__(self):
        self.reset_content()

    def get_macro_arg(self, name):
        for i in range(len(self.__macro_stack) - 1, -1, -1):
            args = self.__macro_stack[i]
            if args.has_key(name):
                return args[name]
        raise ApplicationError('undefined macro argument "%s"' % name)

    def push_macro_args(self, args):
        self.__macro_stack.append(args)

    def pop_macro_args(self):
        args = self.__macro_stack[-1]
        del self.__macro_stack[-1]
        return args

    def push_content_trap(self):
        self.__trap_stack.append(self.__content_parts)
        self.__content_parts = []

    def pop_content_trap(self):
        data = ''.join(self.__content_parts)
        self.__content_parts = self.__trap_stack[-1]
        del self.__trap_stack[-1]
        return data

    def write_content(self, data):
        if type(data) is type(u''):
            data = data.encode('utf-8')
        self.__content_parts.append(data)

    def flush_content(self):
        if self.__trap_stack:
            return
        data = ''.join(self.__content_parts)
        self.send_content(data)
        self.__content_parts = []

    flush_html = flush_content

    def send_content(self, data):
        sys.stdout.write(data)
        sys.stdout.flush()

    def reset_content(self):
        self.__macro_stack = []
        self.__trap_stack = []
        self.__content_parts = []

# ------------------------------------------------------------------
# Template file loaders
# ------------------------------------------------------------------

# A simple template file loader which reads and parses the template
# file every time it is accessed.
class TemplateLoaderMixin:

    '''Basic template loader
    '''
    def __init__(self, base_dir):
        self.__base_dir = base_dir
        self.__loaded_names = {}

    def load_template(self, name):
        self.__loaded_names[name] = 1
        path = os.path.join(self.__base_dir, name)
        try:
            text = open(path).read()
        except IOError, e:
            raise TemplateLoadError("%s: %s" % (path, e.strerror))
        return Template(self, path, text)

    def load_template_once(self, name):
        if self.__loaded_names.get(name):
            return
        return self.load_template(name)


# A caching template file loader which only reloads and parses the
# template file if it has been modified.
class CachingTemplateLoaderMixin:

    '''Caching template file loader
    '''
    def __init__(self, base_dir):
        self.__base_dir = base_dir
        self.__cache = {}

    def load_template(self, name):
        path = os.path.join(self.__base_dir, name)
        try:
            mtime = os.stat(path)[stat.ST_MTIME]
        except OSError, e:
            raise TemplateLoadError("%s: %s" % (path, e.strerror))
        templ = self.__cache.get(path)
        if templ:
            if mtime > templ.__mtime__:
                templ = None
        if not templ:
            try:
                text = open(path).read()
            except IOError, e:
                raise TemplateLoadError(e.strerror)
            templ = Template(self, path, text)
            templ.__mtime__ = mtime
            self.__cache[path] = templ
        return templ

    def load_template_once(self, name):
        path = os.path.join(self.__base_dir, name)
        old_templ = self.__cache.get(path)
        if old_templ:
            old_mtime = old_templ.__mtime__
        new_templ = self.load_template(name)
        if old_templ is None or old_mtime != new_templ.__mtime__:
            return new_templ
        return None

# ------------------------------------------------------------------
# Form element recorders.
# ------------------------------------------------------------------

# Do not record form elements.
class StubRecorderMixin:

    def form_open(self):
        pass

    def form_close(self):
        pass

    def input_add(self, itype, name, value = None, return_list = 0):
        pass

    def merge_request(self):
        for name in self.request.field_names():
            value = self.request.field_value(name)
            self.set_value(name, value)


# Record form element names in a hidden field to allow request
# processing to set fields not supplied to None in execution context.
class Point:

    def __init__(self, x, y):
        self.x = x
        self.y = y

    def __str__(self):
        return '(%s,%s)' % (self.x, self.y)


class NameRecorderMixin:

    NORMAL, LIST, FILE = range(3)

    def __init__(self):
        self.__elem_names = {}

    def form_open(self):
        self.__elem_names = {}
        self.__need_multipart_enc = 0
        self.__needs_close = 1

    def form_close(self):
        if not self.__needs_close:
            raise ApplicationError('<al-form> elements must not be nested')
        self.__needs_close = 0
        text = cPickle.dumps(self.__elem_names, -1)
        text = self.app.pickle_sign(text)
        if have_zlib:
            text = zlib.compress(text)
        text = base64.encodestring(text)
        self.write_content('<input type="hidden" name="__albform__" value="')
        self.write_content(text)
        self.write_content('" />\n')
        self.__elem_names = {}
        return self.__need_multipart_enc

    def input_add(self, itype, name, unused_value = None, return_list = 0):
        if itype == 'file':
            self.__need_multipart_enc = 1
            self.__elem_names[name] = self.FILE
        elif self.__elem_names.has_key(name):
            prev_multiples = self.__elem_names[name]
            implicit_multi = itype in ('radio', 'submit', 'image')
            if not return_list and not implicit_multi:
                raise ApplicationError(
                    'al-input "%s" not defined as "list"' % name)
            if return_list and implicit_multi:
                raise ApplicationError(
                    'al-input "%s" should not be defined as "list"' % name)
            if prev_multiples == 0 and return_list != 0:
                raise ApplicationError(
                    'al-input "%s" initially not defined as "list"' % name)
            if return_list == 0 and prev_multiples != 0:
                raise ApplicationError(
                    'al-input "%s" initially defined as "list"' % name)
        else:
            if return_list:
                self.__elem_names[name] = self.LIST
            else:
                self.__elem_names[name] = self.NORMAL

    def merge_request(self):
        if not self.request.has_field('__albform__'):
            for name in self.request.field_names():
                value = self.request.field_value(name)
                self.set_value(name, value)
            return
        text = self.request.field_value('__albform__')
        text = base64.decodestring(text)
        if have_zlib:
            text = zlib.decompress(text)
        text = self.app.pickle_unsign(text)
        if not text:
            return
        elem_names = cPickle.loads(text)
        for name, mode in elem_names.items():
            if mode == self.FILE:
                value = self.request.field_file(name)
            elif self.request.has_field(name):
                value = self.request.field_value(name)
            else:
                x_name = '%s.x' % name
                y_name = '%s.y' % name
                if self.request.has_field(x_name) \
                   and self.request.has_field(y_name):
                    value = Point(int(self.request.field_value(x_name)),
                                  int(self.request.field_value(y_name)))
                else:
                    value = None
            if mode == self.LIST:
                if not value:
                    value = []
                elif type(value) is not type([]):
                    value = [value]
            self.set_value(name, value)

# ------------------------------------------------------------------
# Handle execution context local namespace - for expression eval().
# ------------------------------------------------------------------

_re_tokens = re.compile(r'([][.])')


class Vars:
    pass


class NamespaceMixin:

    def __init__(self):
        self.locals = Vars()
        self.__globals = {}

    def clear_locals(self):
        self.locals = Vars()

    def set_globals(self, dict):
        self.__globals = dict

    def eval_expr(self, expr):
        self.locals.__ctx__ = self
        try:
            return eval(expr, self.__globals, self.locals.__dict__)
        finally:
            del self.locals.__ctx__

    def set_value(self, name, value):
        if name[0] == '_':
            raise SecurityError('cannot merge %s into namespace' % name)
        # handle iterator back door; op,iter[,value]
        elems = name.split(',')
        if len(elems) >= 2:
            op = elems[0]
            iter = self.get_value(elems[1])
            if iter:
                if len(elems) > 2:
                    # tree operation
                    iter.set_backdoor(op, elems[2], value)
                else:
                    iter.set_backdoor(op, value)
            return
        # handle a[12].b or a.b[12]
        try:
            ID = 0; DOT_OR_OPEN = 1; INDEX = 2; CLOSE = 3
            parent = self.locals
            index = None
            elem = None
            state = ID
            elems = filter(None, _re_tokens.split(name))
            for elem in elems[:-1]:
                if state == ID:
                    parent = getattr(parent, elem)
                    state = DOT_OR_OPEN
                elif state == DOT_OR_OPEN:
                    if elem == '.':
                        state = ID
                    elif elem == '[':
                        state = INDEX
                    else:
                        raise SyntaxError
                elif state == INDEX:
                    index = int(elem)
                    state = CLOSE
                elif state == CLOSE:
                    parent = parent[index]
                    index = None
                    state = DOT_OR_OPEN
            if state == ID:
                setattr(parent, elems[-1], value)
            elif state == CLOSE and elems[-1] == ']' and index is not None:
                parent[index] = value
            else:
                raise SyntaxError
        except:
            try:
                exc_type, exc_value, exc_tb = sys.exc_info()
                try:
                    msg = '    in al-input field "%s"' % name
                    exc_value = '%s\n%s' % (exc_value, msg)
                    if exc_type == IndexError and index is not None:
                        exc_value = '%s (index %d, max %d)' % \
                            (exc_value, index, len(parent))
                    if elem:
                        # Work out where, if possible...
                        ptr = 'near ^'
                        pad = msg.find('"') - len(ptr) + 2
                        for e in elems[:-1]:
                            if e == elem:
                                break
                            pad = pad + len(e)
                        exc_value = '%s\n%s%s' % (exc_value, ' ' * pad, ptr)
                except:
                    pass
                raise exc_type, exc_value, exc_tb
            finally:
                del exc_tb

    def _get_value(self, name):
        # handle iterator back door; op,iter[,value]
        elems = name.split(',')
        if len(elems) >= 2:
            op = elems[0]
            iter = self.get_value(elems[1])
            if len(elems) > 2:
                # tree operation
                return iter.get_backdoor(op, elems[2])
            else:
                return iter.get_backdoor(op)
        # handle a[12].b or a.b[12]
        return eval(name, {}, self.locals.__dict__)

    def make_alias(self, name):
        pos = name.rindex('.')
        value = self._get_value(name[:pos])
        new_name = value.albatross_alias()
        setattr(self.locals, new_name, value)
        self.add_session_vars(new_name)
        return new_name + name[pos:]

    def get_value(self, name):
        try:
            return self._get_value(name)
        except (AttributeError, IndexError, NameError):
            return None

    def has_value(self, name):
        try:
            self._get_value(name)
            return 1
        except (AttributeError, IndexError, NameError):
            return 0

    def has_values(self, *names):
        for name in names:
            if not self.has_value(name):
                return 0
        return 1


# ------------------------------------------------------------------
# Sign pickles to detect tampering at the client side
# ------------------------------------------------------------------

class PickleSignMixin:

    def __init__(self, secret):
        self.__secret = secret

    def pickle_sign(self, text):
        m = md5.new()
        m.update(self.__secret)
        m.update(text)
        text = m.digest() + text
        return text

    def pickle_unsign(self, text):
        digest = text[:16]
        text = text[16:]
        m = md5.new()
        m.update(self.__secret)
        m.update(text)
        if m.digest() == digest:
            return text
        return ''


# ------------------------------------------------------------------
# Session handlers
# ------------------------------------------------------------------

# Nul session handler
class StubSessionMixin:

    def add_session_vars(self, *names):
        pass

    def del_session_vars(self, *names):
        pass

    def encode_session(self):
        return ''

    def load_session(self):
        pass

    def save_session(self):
        pass

    def remove_session(self):
        pass

    def set_save_session(self, flag):
        pass

    def should_save_session(self):
        return 0


class SessionBase:

    def __init__(self):
        self.__init_vars()
        self.__save_session = 1

    def add_session_vars(self, *names):
        if len(names) == 1 and type(names[0]) in (type([]), type(())):
            names = names[0]
        for name in names:
            assert isinstance(name, type(""))
            if not hasattr(self.locals, name):
                raise ApplicationError('add "%s" to locals first' % name)
        for name in names:
            self.__vars[name] = 1

    def default_session_var(self, name, value):
        if not hasattr(self.locals, name):
            setattr(self.locals, name, value)
        self.add_session_vars(name)

    def del_session_vars(self, *names):
        if len(names) == 1 and type(names[0]) in (type([]), type(())):
            names = names[0]
        for name in names:
            if self.__vars.has_key(name):
                del self.__vars[name]

    def session_vars(self):
        return self.__vars.keys()

    def __init_vars(self):
        self.__vars = {'__page__': 1, '__pages__': 1}

    def remove_session(self):
        self.__init_vars()
        self.clear_locals()

    def decode_session(self, text):
        module_path = self.app.module_path()
        if module_path:
            sys.path.insert(0, module_path)
        try:
            try:
                vars = cPickle.loads(text)
            except pickle_errors, e:
                sys.stderr.write('cannot unpickle - %s\n' % e)
                raise ApplicationError("can't unpickle session")
            self.locals.__dict__.update(vars)
            for name in vars.keys():
                self.__vars[name] = 1
        finally:
            if module_path:
                sys.path.remove(module_path)

    def encode_session(self):
        vars = {}
        for name in self.__vars.keys():
            if hasattr(self.locals, name):
                vars[name] = getattr(self.locals, name)
        try:
            return cPickle.dumps(vars, -1)
        except pickle_errors, e:
            for name, value in vars.items():
                try:
                    cPickle.dumps(value, -1)
                except pickle_errors, e:
                    raise ApplicationError('locals "%s": %s' % (name, e))
            raise ApplicationError('cannot pickle ctx.locals: %s' % e)

    def set_save_session(self, flag):
        self.__save_session = flag

    def should_save_session(self):
        return self.__save_session


# Session in hidden fields
class HiddenFieldSessionMixin(SessionBase):

    def encode_session(self):
        text = SessionBase.encode_session(self)
        text = self.app.pickle_sign(text)
        if have_zlib:
            text = zlib.compress(text)
        return base64.encodestring(text)

    def load_session(self):
        if not self.request.has_field('__albstate__'):
            return
        text = self.request.field_value('__albstate__')
        text = base64.decodestring(text)
        if have_zlib:
            text = zlib.decompress(text)
        text = self.app.pickle_unsign(text)
        if text:
            SessionBase.decode_session(self, text)

    def save_session(self):
        pass

    def form_close(self):
        if self.should_save_session():
            text = self.encode_session()
            self.write_content('<input type="hidden" name="__albstate__" value="')
            self.write_content(text)
            self.write_content('" />\n')


# ------------------------------------------------------------------
# An execution context for stand-alone template file use.
# ------------------------------------------------------------------

def _caller_globals(name):
    try:
        raise ZeroDivisionError
    except ZeroDivisionError:
        frame = sys.exc_traceback.tb_frame
    while frame.f_code.co_name != name:
        frame = frame.f_back
    while frame.f_code.co_name == name:
        frame = frame.f_back
    return frame.f_globals


class SimpleContext(NamespaceMixin, ExecuteMixin, ResourceMixin,
                    TemplateLoaderMixin, StubRecorderMixin, StubSessionMixin):

    def __init__(self, template_path):
        from albatross import tags
        NamespaceMixin.__init__(self)
        ExecuteMixin.__init__(self)
        ResourceMixin.__init__(self)
        TemplateLoaderMixin.__init__(self, template_path)
        apply(self.register_tagclasses, tags.tags)
        self.set_globals(_caller_globals('__init__'))



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# HiddenFieldSession functionality tests
#
# $Id: ses_hiddenfield.py,v 1.5 2002/07/30 07:33:51 andrewm Exp $
#

import os, sys

import unittest
import albatross

import request

class Vars:
    pass

class HiddenFieldSessionApp(albatross.PickleSignMixin):
    def module_path(self):
        pass

class HiddenFieldSession(albatross.HiddenFieldSessionMixin):
    """ 
    This corresponds to the object albatross users know as the ctx,
    although since we aren't using the normal set of mixins, we have to
    supply our own locals and request members.
    """
    def __init__(self, request):
        self.app = HiddenFieldSessionApp(secret="test")
        self.request = request
        self.locals = Vars()
        albatross.HiddenFieldSessionMixin.__init__(self)

    def test_save(self):
        self.form_close()
        self.request.flush_content()

    def write_content(self, text):
        self.request.write_content(text)

class Session:
    """
    This represents a virtual "user" - essentially just a place to store
    their session id while the ctx objects come and go.
    """
    def __init__(self):
        self.request = request.DummyFieldRequest()

    def new_context(self):
        return HiddenFieldSession(self.request)

class HiddenFieldCase(unittest.TestCase):
    def setUp(self):
        pass

    def tearDown(self):
        pass

    def new_session(self):
        return Session()



import sys
import os
import getopt

sys.path.insert(0, '..')

_ARGS_ARGS_FLAG = 4
_KW_ARGS_FLAG = 8


def find_methods(klass):
    methods = []
    for name, attr in klass.__dict__.iteritems():
        if name.startswith('_'):
            continue
        if callable(attr):
            args = []
            code = attr.func_code
            arg_names = list(code.co_varnames[1:code.co_argcount])
            arg_idx = code.co_argcount
            if code.co_flags & _ARGS_ARGS_FLAG:
                arg_names.append('*' + code.co_varnames[arg_idx])
                arg_idx += 1
            if code.co_flags & _KW_ARGS_FLAG:
                arg_names.append('**' + code.co_varnames[arg_idx])
            methods.append((name, arg_names, klass.__name__))
    for base in klass.__bases__:
        methods.extend(find_methods(base))
    return methods


def combine_methods(methods):
    combined = []
    names = {}
    for name, arg_names, klass in methods:
        if names.get(name):
            continue
        combined.append((name, arg_names, klass))
        names[name] = 1
    combined.sort()
    return combined


def method_table(klass):
    n = find_methods(klass)
    print r'\begin{longtableii}{l|l}{textrm}{Method}{Mixin}'
    for name, arg_names, klass in combine_methods(n):
        args = ', '.join(arg_names)
        print r'\lineii{\method{%s(%s)}}{\class{%s}}' % (name, args, klass)
    print r'\end{longtableii}'


def usage():
    sys.exit('usage: %s [-o filename] module.class'
             % os.path.basename(sys.argv[0]))


if __name__ == '__main__':
    try:
        opts, args = getopt.getopt(sys.argv[1:], 'o:', ['output='])
    except getopt.GetoptError:
        usage()
    for opt, arg in opts:
        if opt in ('-o', '--output'):
            sys.stdout = open(arg, 'w')

    if len(args) != 1:
        usage()
    try:
        mod_name, klass_name = args[0].split('.')
    except:
        usage()

    mod = __import__(mod_name)
    klass = getattr(mod, klass_name)
    method_table(klass)



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Test macros.
#
# $Id: macro.py,v 1.10 2003/01/24 11:34:31 andrewm Exp $

import unittest
import albatross
import albatross_test

class MacroCase(albatross_test.AlbatrossTestCase):
    template_dir = "macro"

    def check_macro_0arg(self):
        self.html_test("0arg.html", "content\n")

    def check_macro_0arg_dup(self):
        self.html_test("0arg-dup.html", "content\ncontent\n")

    def check_macro_1arg(self):
        self.html_test("1arg.html", "xxx argument xxx\n")

    def check_macro_2arg(self):
        self.html_test("2arg.html", "xxx first second xxx\n")

    def check_macro_unclosed(self):
        self.assertRaises(albatross.ApplicationError, self.html_test, "unclosed.html", "(none)")

    def check_macro_unclosedarg(self):
        self.assertRaises(albatross.ApplicationError, self.html_test, "unclosedarg.html", "(none)")

    def check_macro_missingarg(self):
        self.assertRaises(albatross.ApplicationError, self.html_test, "missingarg.html", "(none)")

class MacroSuite(unittest.TestSuite):
    test_list = (
        "check_macro_0arg",
        "check_macro_0arg_dup",
        "check_macro_1arg",
        "check_macro_2arg",
        "check_macro_unclosed",
        "check_macro_unclosedarg",
        "check_macro_missingarg",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(MacroCase, self.test_list))

def suite():
    return MacroSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/popview4', cgi = 1)
i.add_exec('popview.py')
i.add_module('popviewlib.py')
i.add_module('login.py')
i.add_html('login.html')
i.add_module('list.py')
i.add_html('list.html')
i.add_module('detail.py')
i.add_html('detail.html')
i.install()



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Test template expansion.
#
# $Id: lookup.py,v 1.12 2003/01/24 11:34:31 andrewm Exp $

import unittest
import albatross
import albatross_test

class TemplateTestCase(albatross_test.AlbatrossTestCase):
    template_dir = "lookup"
    lookup_cases = (
        (0,	"Zero\n"),
        (1,	"One\n"),
        (2,	"Two\n"),
        (3,	"Many\n"),
        (4,	"Many\n"),
        (-1,	"Many\n"),
        (20,	"Many\n")
    )

    def html_test(self, template, value, expect):
        self.ctx.locals.i = value
        albatross_test.AlbatrossTestCase.html_test(self, template, expect)


class FileTemplateCase(TemplateTestCase):
    def check_enum(self):
        for (x, result) in self.lookup_cases:
            self.html_test("lookup.html", x, result)

    def check_no_default(self):
        no_default_cases = (
            (0,	"Zero\n"),
            (1,	"One\n"),
            (2,	"\n"),
            (20, "\n"),
        )
        for (x, result) in no_default_cases:
            self.html_test("no-default.html", x, result)

    def check_no_close(self):
        self.assertRaises(albatross.ApplicationError, self.html_test, "no-close.html", 0, "(none)")

# This differs from the previous class because it generates the html output by
# feeding in a string to the templating code.

class StringTemplateCase(TemplateTestCase):
    template_string = '''
<al-lookup name="xlat">
 <al-item expr="0">Zero</al-item>
 <al-item expr="1">One</al-item>
 <al-item expr="2">Two</al-item>
 Many</al-lookup>
<al-value expr="i" lookup="xlat" whitespace>
'''
    # this overrides the albatross_test.Albatross.gen_html method.
    def gen_html(self, ctx, template_string):
        ctx.push_content_trap()
        tmpl = albatross.template.Template(ctx, '<ether>', template_string)
        tmpl.to_html(ctx)
        return ctx.pop_content_trap()

    def check_enum(self):
        for (x, result) in self.lookup_cases:
            self.html_test(self.template_string, x, result)


class TemplateSuite(unittest.TestSuite):
    test_list = (
        "check_enum",
        "check_no_default",
        "check_no_close",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(FileTemplateCase, self.test_list))
        self.addTest(StringTemplateCase("check_enum"))

def suite():
    return TemplateSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')





#
# Copyright 2003 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# AUTHOR(S)
#       Matt Goodall <matt AT pollenation DOT net>

import BaseHTTPServer
import cgi
import mimetypes
from cStringIO import StringIO
from urlparse import urlsplit
import os.path
import shutil
import time
import stat

from albatross.cgiapp import RequestFields

RFC822_FORMAT = '%a, %d %b %Y %H:%M:%S +0000'

class Request(RequestFields):

    """BaseHTTPServer request adaptor"""

    def __init__(self, req):

        self._req = req
        self._status = 200;
        self.headers_out = {}
        self.data = StringIO()

        fields = self.make_fields()
        RequestFields.__init__(self, fields)

    def make_fields(self):
        '''
        Make a cgi.FieldStorage from the request.

        This feels like a complete hack but it makes good use of the standard
        cgi module ... and it works! Basically, we're pretending to be a
        CGI by providing an "environment" dictionary with everything the
        cgi.FieldStorage needs (I simply read the code to know what's used).
        '''
        
        environ = {}
        environ['REQUEST_METHOD'] = self._req.command
        environ['QUERY_STRING'] = urlsplit(self._req.requestline.split()[1])[3]

        self._req.headers.setdefault(
            'content-type',
            'application/x-www-form-urlencoded')

        field_storage = cgi.FieldStorage(
            fp=self._req.rfile,
            headers=self._req.headers,
            environ=environ)

        return field_storage
        
    def get_header(self, name):
        return self._req.headers.get(name)
        
    def write_header(self, name, value):
        self.headers_out[name] = value

    def end_headers(self):
        pass

    def get_uri(self):
        return self._req.path

    def get_path_info(self):
        return self._req.path

    def get_servername(self):
        server_name = self._req.server.server_name
        port = self._req.server.server_port
        return '%s:%d' % (server_name, port)

    def write_content(self, data):
        self.data.write(data)

    def redirect(self, loc):
        self.write_header('Location', loc)
        self.write_header('Status', '301 Moved Permanently')
        self._status = 301
        self.end_headers()

    def set_status(self, status):
        self._status = status

    def status(self):
        return self._status


class RequestHandler(BaseHTTPServer.BaseHTTPRequestHandler):

    """
    Handle HTTP request, routing them through to the Albatross
    application
    """

    def do_GET(self):
        # Try the static resources first
        if self.server.static_resources:
            for uri_path, fs_path in self.server.static_resources.items():
                if self.path.startswith(uri_path):
                    path = self.path.split(uri_path)[1]
                    self.send_file('%s%s' % (fs_path, path))
                    return
        self.process_request()

    def do_POST(self):
        self.process_request()

    def process_request(self):
        '''Process a request to the Albatross app'''
        self.log_request()
        req = Request(self)
        self.server.app.run(req)
        self.send_response(req.status())
        for name, value in req.headers_out.items():
            self.send_header(name, value)
        self.end_headers()
        if req.status() == 200:
            self.wfile.write(req.data.getvalue())

    def send_file(self, path):
        '''Send a file directly from the filesystem'''
        
        self.log_request()

        # Stat the file
        try:
            stats = os.stat(path)
        except OSError, e:
            # 404 - file not found
            self.send_error(404)
            self.end_headers()
            return

        # Get the last modified date in RFC822 form
        last_modified = time.gmtime(stats[stat.ST_MTIME])
        last_modified = time.strftime(RFC822_FORMAT, last_modified)

        # See if a 304 (not modified) is enough
        ts = self.headers.get('If-Modified-Since')
        if ts and ts == last_modified:
            self.send_response(304)
            self.end_headers()
            return
        
        # Need to send the whole file
        f = None
        try:
            try:
                f = file(path, 'rb')
                self.send_response(200)
                self.send_header('Content-Type', mimetypes.guess_type(path)[0])
                self.send_header('Content-Length', os.path.getsize(path))
                self.send_header('Last-Modified', last_modified)
                self.end_headers()
                shutil.copyfileobj(f, self.wfile)
            except IOError, e:
                self.send_error(500)
                self.end_headers()
        finally:
            if f:
                f.close()


class HTTPServer(BaseHTTPServer.HTTPServer):

    """
    Simple, standalone HTTP server for Albatross applications.
    
    HTTPServer is a simple web server dedicated to processing requests and
    mapping that request through to Albatross. It can also serve static
    resources such as images, stylesheets etc directly from the filesystem.
    """

    def __init__(self, app, port, static_resources=None):
        '''
        Create an HTTP server for the Albatross application, app,
        and listen for requests on the specified port.

        static_resources is an optional dictionary that maps uri paths to
        a corresponding filesystem path. Static resources are served directly
        from the filesystem without involving the Albatross application.
        '''
        BaseHTTPServer.HTTPServer.__init__( self, ('', port), RequestHandler)
        self.app = app
        self.static_resources = static_resources



#!/usr/bin/python
import cgi
from albatross import SimpleContext

form = cgi.FieldStorage()
ctx = SimpleContext('.')

ctx.locals.option_list1 = ['option1', 'option2', 'option3']
ctx.locals.option_list2 = [(1, 'option1'), (2, 'option2'), (3, 'option3')]

ctx.locals.form = form
for name in form.keys():
    if type(form[name]) is type([]):
        value = []
        for elem in form[name]:
            value.append(elem.value)
    else:
        value = form[name].value
    setattr(ctx.locals, name, value)
templ = ctx.load_template('form.html')
templ.to_html(ctx)

print 'Content-Type: text/html'
print
ctx.flush_content()



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Build and test all listed modules.
#
# The module files must have a function "suite" which returns
# an instance of unittest.TestSuite or unittest.TestCase.
#
# $Id: all.py,v 1.11 2002/07/30 07:33:51 andrewm Exp $

import unittest

class AllTestSuite(unittest.TestSuite):
    # add modules with tests here
    all_tests = [
    	"basic",
    	"control_flow",
        "env",
        "form",
        "include",
        "iterator",
        "lookup",
        "macro",
        "tree",
        "whitespace"
    ]
    def __init__(self):
        unittest.TestSuite.__init__(self)
        for m in map(__import__, self.all_tests):
            self.addTest(m.suite())

if __name__ == '__main__':
    unittest.main(defaultTest='AllTestSuite')



#!/usr/bin/python
import time
import calendar
import albatross
from albatross import SimpleApp
from albatross.cgiapp import Request


class Calendar(albatross.EmptyTag):

    name = 'alx-calendar'

    def to_html(self, ctx):
        year = self.get_attrib('year')
        if year is not None:
            year = ctx.eval_expr(year)
        month = self.get_attrib('month')
        if month is not None:
            month = ctx.eval_expr(month)
        if month is None or year is None:
            now = time.localtime(time.time())
            if year is None:
                year = now[0]
            if month is None:
                month = now[1]
        ctx.write_content('<table>\n')
        ctx.write_content('<tr align="center"><td colspan="7">%s %s</td></tr>\n' \
                          % (calendar.month_name[month], year))
        ctx.write_content('<tr>')
        for i in range(7):
            ctx.write_content('<td>%s</td>' \
                              % calendar.day_abbr[(i + 6) % 7][:2])
        ctx.write_content('</tr>\n')
        calendar.setfirstweekday(6)
        for r in calendar.monthcalendar(year, month):
            ctx.write_content('<tr align="right">')
            for i in range(7):
                if r[i]:
                    ctx.write_content('<td>%s</td>' % r[i])
                else:
                    ctx.write_content('<td></td>')
            ctx.write_content('</tr>\n')
        ctx.write_content('</table>\n')


class CalendarPage:

    def page_display(self, ctx):
        ctx.locals.year = 2002
        ctx.run_template('cal.html')


app = SimpleApp(base_url='cal.py',
                template_path='.',
                start_page='start',
                secret='-=-secret-=-')
app.register_tagclasses(Calendar)
app.register_page('start', CalendarPage())

if __name__ == '__main__':
    app.run(Request())



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Base class for testing Albatross functionality.
#
# Sub-classes are expected to define a class variable "template_dir" which
# tells albatross which directory to search for the html template files.
#
# There is also a class variable, self.ctx, available to classes derived
# from this which contains the current Albatross execution context.
#
# $Id: albatross_test.py,v 1.9 2002/07/30 07:33:51 andrewm Exp $

import unittest
import albatross

import sys
import os

class AlbatrossTestError(Exception):
    pass

app = albatross.Application('here')

class TestContext(albatross.TemplateLoaderMixin,
                  albatross.AppContext, albatross.NameRecorderMixin,
                  albatross.StubSessionMixin):
    def __init__(self, template_path):
        albatross.TemplateLoaderMixin.__init__(self, template_path)
        albatross.AppContext.__init__(self, app)
        albatross.NameRecorderMixin.__init__(self)

    def current_url(self):
        return 'here'

class AlbatrossTestCase(unittest.TestCase):
    def setUp(self):
        if not hasattr(self, "template_dir"):
            raise AlbatrossTestError("%s: template_dir not defined in class" % self.__class__)

        # This is because we may not be imported from the current directory
        base_dir = os.path.dirname(sys.modules[__name__].__file__)
        template_dir = os.path.join(base_dir, self.template_dir)

        self.ctx = TestContext(template_dir)

    def tearDown(self):
        pass

    # we use the content_trap to capture and return generated html;
    # html_flush() writes it to stdout directly.
    def gen_html(self, ctx, template):
        ctx.push_content_trap()
        tmpl = ctx.load_template(template)
        tmpl.to_html(ctx)
        return ctx.pop_content_trap()

    def html_test(self, template_file, expect):
        result = self.gen_html(self.ctx, template_file)
        self.assertEqual(result, expect, 'expected "%s", got "%s"' % (expect, result))



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
from albatross.common import *
from albatross import tags
from albatross.template import *
from albatross.context import *
from albatross.session import *
from albatross.app import *
from albatross.randompage import *
from albatross.sessionfile import *

ListIterator = tags.ListIterator
TreeIterator = tags.TreeIterator
LazyTreeIterator = tags.LazyTreeIterator
EllipsisTreeIterator = tags.EllipsisTreeIterator

__version__ = '1.11'



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
import sys
import os
from errno import *
import signal
import time


class PidFileError(Exception):
    pass


class PidFile:

    def __init__(self, pidfilename):
        self._pid_filename = pidfilename

    def getpid(self):
        try:
            pid = open(self._pid_filename,"r").readline()
        except IOError, (eno, estr):
            if eno == ENOENT:
                return 0
            raise PidFileError("%s: %s" % (self._pid_filename, estr))
        try:
            pid = int(pid)
        except ValueError:
            raise PidFileError("%s: unknown format" % \
                               self._pid_filename)
        if pid <= 1:
            raise PidFileError("%s: unknown format" % \
                               self._pid_filename)
        try:
            os.kill(pid,0)
        except OSError, (eno, estr):
            if eno == ESRCH:
                # Stale lock file, remove and sleep to avoid creation race
                sys.stderr.write("Removing state lock for pid %d\n" % pid)
                os.unlink(self._pid_filename)
                time.sleep(2)
                return 0
            raise PidFileError("pid %d: %s" % (pid, estr))

        return pid

    def is_running(self):
        return (self.getpid() != 0)

    def kill(self, sig = signal.SIGTERM):
        pid = self.getpid()
        if pid:
            os.kill(pid, sig)

    def start(self, pid = None):
        if pid is None:
            pid = os.getpid()

        daemon_pid = self.getpid()
        if daemon_pid:
            raise PidFileError("daemon already running, pid %d" % daemon_pid)

        # We use fd ops because we want O_EXCL semantics on the created file
        try:
            pid_fd = os.open(self._pid_filename, 
                             os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0644)
        except OSError, (eno, estr):
            raise PidFileError("can't create pid file %s: %s" %\
                               (self._pid_filename, estr))
        os.write(pid_fd, "%d\n" % pid)
        os.close(pid_fd)

    def stop(self):
        pid = self.getpid()
        if pid != os.getpid():
            raise PidFileError("Pid file doesn't belong to us - pid %d" % pid)
        try:
            os.unlink(self._pid_filename)
        except:
            pass



import string
import poplib

pophost = 'pop'


class Headers:

    def __init__(self):
        self.hdrs = {}

    def append(self, header):
        if not header:
            return
        parts = string.split(header, ': ', 1)
        name = string.capitalize(parts[0])
        if len(parts) > 1:
            value = parts[1]
        else:
            value = ''
        curr = self.hdrs.get(name)
        if not curr:
            self.hdrs[name] = value
            return
        if type(curr) is type(''):
            curr = self.hdrs[name] = [curr]
        curr.append(value)

    def __getitem__(self, name):
        return self.hdrs.get(string.capitalize(name), '')


class Msg:

    def __init__(self, mbox, msgnum):
        self.mbox = mbox
        self.msgnum = msgnum
        self.read_headers()

    def read_headers(self):
        res = self.mbox.top(self.msgnum, 0)
        hdrs = Headers()
        hdr = None
        for line in res[1]:
            if line and line[0] in string.whitespace:
                hdr = hdr + '\n' + line
            else:
                hdrs.append(hdr)
                hdr = line
        hdrs.append(hdr)
        self.hdrs = hdrs
        return hdrs

    def read_body(self):
        res = self.mbox.retr(self.msgnum)
        lines = res[1]
        for i in range(len(lines)):
            if not lines[i]:
                break
        self.body = string.join(lines[i:], '\n')
        return self.body


class Mbox:

    def __init__(self, name, passwd):
        self.mbox = poplib.POP3(pophost)
        self.mbox.user(name)
        self.mbox.pass_(passwd)

    def __getitem__(self, i):
        try:
            return Msg(self.mbox, i + 1)
        except poplib.error_proto:
            raise IndexError

    def __len__(self):
        len, size = self.mbox.stat()
        return len



from utils import Node

def page_display(ctx):
    ctx.run_template('tree.html')


def page_process(ctx):
    if ctx.req_equals('paginate'):
        ctx.redirect('paginate')
    if not hasattr(ctx.locals, 'tree'):
        ctx.locals.tree \
            = Node('a', [Node('a', [Node('a'),
                                    Node('b')]),
                         Node('b', [Node('a', [Node('a', [Node('a'),
                                                          Node('b')])]),
                                    Node('b'),
                                    Node('c', [Node('a'),
                                               Node('b')])])])
        ctx.add_session_vars('tree')



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Test if-elif-else flow constructs.
#
# $Id: control_flow.py,v 1.9 2003/01/24 11:34:31 andrewm Exp $

import unittest
import albatross
import albatross_test

class IfThenElifElseCase(albatross_test.AlbatrossTestCase):
    template_dir = "control_flow"

    def html_test(self, template, value, expect):
        self.ctx.locals.x = value
        albatross_test.AlbatrossTestCase.html_test(self, template, expect)

    def check_if(self):
        self.html_test("if-elif-else.html", 0, "zero\n")

    def check_elif(self):
        self.html_test("if-elif-else.html", 1, "positive\n")

    def check_else(self):
        self.html_test("if-elif-else.html", -1, "negative\n")

    def check_unclosed(self):
        self.assertRaises(albatross.ApplicationError, self.html_test, "if-elif-fail.html", 0, "(none)")


class ControlFlowSuite(unittest.TestSuite):
    test_list = (
        "check_if",
        "check_elif",
        "check_else",
        "check_unclosed"
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(IfThenElifElseCase, self.test_list))

def suite():
    return ControlFlowSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# Basic functionality tests - save and restore a single context
#
# $Id: basic.py,v 1.3 2002/07/30 07:33:51 andrewm Exp $
#

import unittest

import ses_file
import ses_server
import ses_hiddenfield

class BasicCase:
    def runTest(self):

        test_str = "Green Eggs and Ham"

        session = self.new_session()

        ctx = session.new_context()
        ctx.load_session()
        self.assertEqual(ctx.locals.__dict__, {})
        ctx.locals.test_var = test_str
        ctx.add_session_vars('test_var')
        ctx.test_save()

        ctx = session.new_context()
        ctx.load_session()
        self.assertEqual(ctx.locals.__dict__.keys(), ['test_var'])
        self.assertEqual(ctx.locals.test_var, test_str)


class FileBasicCase(ses_file.FileCase, BasicCase):
    pass

class ServerBasicCase(ses_server.ServerCase, BasicCase):
    pass

class HiddenFieldBasicCase(ses_hiddenfield.HiddenFieldCase, BasicCase):
    pass


class BasicCase(unittest.TestCase):
    def check_file(self):
        self.test_saverestore(ses_file.Session())

    def check_server(self):
        self.test_saverestore(ses_server.Session())

    def check_hiddenfield(self):
        self.test_saverestore(ses_hiddenfield.Session())


class BasicSuite(unittest.TestSuite):
    test_list = [
        FileBasicCase(),
        ServerBasicCase(),
        HiddenFieldBasicCase(),
    ]

    def __init__(self):
        unittest.TestSuite.__init__(self, self.test_list)

def suite():
    return BasicSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='BasicSuite')



#!/usr/bin/python
import os, stat
from albatross.cgiapp import Request
from albatross import SimpleApp


class Node:

    def __init__(self, ctx, name, path):
        self.name = name
        self.path = path
        st_mode = os.stat(path)[0]
        if stat.S_ISDIR(st_mode):
            self.children = []
            self.children_loaded = 0

    def load_children(self, ctx):
        names = os.listdir(self.path)
        names.sort()
        self.children = []
        for name in names:
            if name in ('.', '..'):
                continue
            self.children.append(Node(ctx, name, os.path.join(self.path, name)))

    def albatross_alias(self):
        return self.path


class TreePage:

    def page_display(self, ctx):
        ctx.locals.tree = Node(ctx, '/', '/var/www')
        ctx.run_template('tree.html')


app = SimpleApp(base_url='tree.py',
                template_path='.',
                start_page='start',
                secret='-=-secret-=-')
app.register_page('start', TreePage())


if __name__ == '__main__':
    app.run(Request())



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#

import urlparse

from albatross.template import Template
from albatross.context import CachingTemplateLoaderMixin, PickleSignMixin,\
     _caller_globals
from albatross.app import Application, PageModuleMixin, SimpleAppContext,\
     SessionAppContext
from albatross.session import SessionServerAppMixin


class RandomPageModuleMixin(PageModuleMixin):

    def load_page(self, ctx):
        # Get page name from request URI
        uri = ctx.request.get_uri()
        page = self.get_page_from_uri(ctx, uri)
        if not page:
            ctx.redirect(self.start_page())

        # Try to load the page module
        self.__page_found = 1
        try:
            self.load_page_module(ctx, page)
        except ImportError, e:
            if not str(e).endswith(page):
                raise
            ctx.locals.page_name = page
            self.__page_found = 0
        if self.__page_found:
            self.page_enter(ctx)

    def get_page_from_uri(self, ctx, uri):
        try:
            base_path = urlparse.urlparse(self.base_url())[2]
            uri_path = urlparse.urlparse(uri)[2]
            # Fixup base_path to end with a '/' so that it can be used as
            # a root path.
            if not base_path.endswith('/'):
                base_path =  base_path + '/'
            return uri_path.split(base_path, 1)[1]
        except IndexError:
            return None

    def load_badurl_template(self, ctx):
        return Template(ctx, '<internal>',
'''<!DOCTYPE HTML PUBLIC "-//IETF//DTD HTML 2.0//EN">
<html>
 <head>
  <title>404 Not Found</title>
 </head>
 <body>
  <h1>Oops!</h1>
   URL <al-value expr="page_name"> was not found on this server.<p>
 </body>
</html>
''')

    def page_enter(self, ctx):
        if self.__page_found:
            return PageModuleMixin.page_enter(self, ctx, ())
        return None

    def process_request(self, ctx):
        if self.__page_found:
            return PageModuleMixin.process_request(self, ctx)
        return None

    def display_response(self, ctx):
        if self.__page_found:
            PageModuleMixin.display_response(self, ctx)
        else:
            # Display error page - no page module so use the mainline
            # globals
            templ = self.load_badurl_template(ctx)
            ctx.set_globals(_caller_globals('run'))
            templ.to_html(ctx)

class RandomModularApp(PickleSignMixin,
                       Application,
                       CachingTemplateLoaderMixin,
                       RandomPageModuleMixin):
    def __init__(self, base_url, page_path, start_page, secret):
        PickleSignMixin.__init__(self, secret)
        Application.__init__(self, base_url)
        CachingTemplateLoaderMixin.__init__(self, page_path)
        RandomPageModuleMixin.__init__(self, page_path, start_page)

    def create_context(self):
        return SimpleAppContext(self)


# Same as RandomModularApp with server-side session support
class RandomModularSessionApp(PickleSignMixin,
                              Application,
                              CachingTemplateLoaderMixin,
                              RandomPageModuleMixin,
                              SessionServerAppMixin):

    def __init__(self, base_url, page_path, start_page, secret,
                 session_appid, session_server = 'localhost', server_port = 34343, session_age = 1800):
        PickleSignMixin.__init__(self, secret)
        Application.__init__(self, base_url)
        CachingTemplateLoaderMixin.__init__(self, page_path)
        RandomPageModuleMixin.__init__(self, page_path, start_page)
        SessionServerAppMixin.__init__(self, session_appid, session_server, server_port, session_age)

    def create_context(self):
        return SessionAppContext(self)



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# Templating core functionality
#
import re

from albatross.common import *

# The template file parser treats HTML files as plain text with
# embedded template tags.  In parsing the file a tag tree is
# constructed.  Tags are either empty or enclosing.


# Plain text from the template file is managed by a Text object.  This
# allows the template interpreter to treat text no differently from a
# tag.

class Text:

    name = None

    def __init__(self, text):
        self.text = text

    def __repr__(self):
        return 'Text(%s)' % self.text

    def to_html(self, ctx):
        ctx.write_content(self.text)


# All tags in the toolkit and application are implemented by
# subclassing either EmptyTag or EnclosingTag.  Each of these classes
# inherit from Tag.

class Tag:

    '''Functionality common to all HTML template tags.
    '''

    def __init__(self, ctx, filename, line_num, attribs):
        '''Store where the tag was defined for execution trace
        generation
        '''
        self.filename = filename
        self.line_num = line_num
        self.attribs = attribs
        self.attrib_order = None

    def __repr__(self):
        parts = []
        parts.append('<')
        parts.append(self.name)
        for name in self.attrib_order:
            value = self.attribs[name]
            if value is not None:
                parts.append(' %s="%s"' % (name, value))
            else:
                parts.append(' %s' % name)
        parts.append('>')
        return ''.join(parts)

    def raise_error(self, msg):
        raise ApplicationError('%s:%s: %s' % \
                               (self.filename, self.line_num, msg))

    def has_attrib(self, name):
        '''Was the named attribute named with the tag
        '''
        return self.attribs.has_key(name)

    def assert_has_attrib(self, name):
        if not self.attribs.has_key(name):
            self.raise_error('missing "%s" attribute' % name)

    def get_attrib(self, name, default = None):
        '''Return the value of the named attribute
        '''
        return self.attribs.get(name, default)

    def set_attrib_order(self, order):
        self.attrib_order = order

    def set_attrib(self, name, value):
        '''Set the value of the named attribute
        '''
        self.attribs[name] = value
        if name not in self.attrib_order:
            self.attrib_order.append(name)

    def attrib_items(self):
        return self.attribs.items()

    def write_attribs_except(self, ctx, *exclude):
        for name in self.attrib_order:
            if name in exclude:
                continue
            value = self.attribs[name]
            if value is not None:
                ctx.write_content(' %s="%s"' % (name, value))
            else:
                ctx.write_content(' %s' % name)


# Tags such as <al-input>, <al-let>, <al-usearg> do not enclose
# content.  These tags just use the information in their attributes to
# generate output.
# 
# When you subclass the EmptyTag class you must supply a name class
# attribute and you should almost certainly override the to_html()
# method.
# 
# class HR(template.EmptyTag):
#     name = 'al-hr'
#
#     def to_html(self, ctx):
#         ctx.write_content('<hr noshade>')
#
# The name attribute allows the code which interprets the parsed
# template to identify each tag.  It is also used by the parser to
# locate which class to instantiate when it finds a tag of the same
# name in the template file.

class EmptyTag(Tag):

    '''All tags which do not enclose content inherit from this class
    '''

    def has_content(self):
        return 0

    def to_html(self, ctx):
        pass


# Content objects are the basic building block of the parsed HTML
# template.  Converting a Content object to HTML simply converts each
# item in the object to HTML.  All content enclosing tags use Content
# objects to store their content and then use the special tag logic to
# control when and how often the content should be sent as output.

class Content:

    '''Manage a list of content items.
    '''

    def __init__(self):
        self.items = []

    def __repr__(self):
        return 'Content(%s)' % self.items

    def append(self, item):
        self.items.append(item)

    def to_html(self, ctx):
        for item in self.items:
            item.to_html(ctx)

# Tags such as <al-in>, <al-form>, <al-tree>, <al-expand> do
# enclose content.
# 
# When you subclass the EnclosingTag class you must supply a name
# class attribute and you should almost certainly override the
# to_html() method.  For example, the following is very simple if:
# 
# class If(template.ContentTag):
#     name = 'al-if'
#
#     def __init__(self, ctx, filename, line_num, attribs):
#         template.ContentTag.__init__(self, ctx, filename, line_num, attribs)
#         if not self.has_attrib('expr'):
#             self.raise_error('missing "expr" attribute')
#         self.expr = compile(self.get_attrib('expr'), '<albatross>', 'eval')
#
#     def to_html(self, ctx):
#         if ctx.eval_expr(self.expr):
#             template.ContentTag.to_html(self, ctx)

class EnclosingTag(Tag):

    '''All tags which enclose content inherit from this class
    '''

    def __init__(self, ctx, filename, line_num, attribs):
        Tag.__init__(self, ctx, filename, line_num, attribs)
        self.content = Content()

    def __repr__(self):
        return Tag.__repr__(self) + repr(self.content) + '</%s>' % self.name

    def has_content(self):
        return 1

    def append(self, item):
        self.content.append(item)

    def to_html(self, ctx):
        self.content.to_html(ctx)


# A Template object contains the parsed HTML content of a single
# template file.  Loading a template parses the HTML and compiles the
# expressions within the file.  The execution phase happens later once
# the application has established an execution context.
# 
# A tag "manager" is passed in which resolves the tag name to a Python
# class which implements that tag.  This allows the application to
# extend the standard tag set without changing the toolkit.

# Parsing works by splitting the file into a stream of text, tag,
# text, tag, ...  The current parent object for content is maintained
# on a stack, each time a new content enclosing tag is encountered,
# the current parent is saved and all new content is directed to the
# new parent.  When the close tag is encountered the old parent is
# popped off the stack.

# Locate tags; remember start, attribs, end
_sre_dblquote_nogrp = r'"(?:[^"\\]*(?:\\.[^"\\]*)*)"'   # eg, "hello \"bill\""
_sre_sglquote_nogrp = r"'(?:[^'\\]*(?:\\.[^'\\]*)*)'"   # eg, 'sam\'s ball'
_sre_attr = r'(?:' + \
    r'\s+\w+=' + _sre_dblquote_nogrp + '|' + \
    r'\s+\w+=' + _sre_sglquote_nogrp + '|' + \
    r'\s+\w+' + ')'                                     # eg, attr="value"
_sre_attrs = r'(' + _sre_attr + r'*)'                   # eg, a1="v1" a2="v2"
_sre_tagname = r'(/?alx?-\w+)'                          # eg, al-value
_sre_tag = r'<'+_sre_tagname+_sre_attrs+r'(\s*/?>)'     # a full albatross tag
_re_tag = re.compile(_sre_tag, re.I|re.M|re.S)

_re_leading = re.compile(r'^\s+')

# Extract attributes
_sre_dblquote_grp = r'"([^"\\]*(?:\\.[^"\\]*)*)"'
_sre_sglquote_grp = r"'([^'\\]*(?:\\.[^'\\]*)*)'"
_sre_attr_grp = r'(\w+)=(?:'+_sre_dblquote_grp+'|'+_sre_sglquote_grp+')|(\w+)'
_re_attrib = re.compile(_sre_attr_grp, re.I|re.M)

_re_dequote = re.compile(r'\\(.)')


class Template:

    '''Manage a single HTML template file
    '''

    def __init__(self, ctx, filename, text):
        '''The ctx argument is is used to resolve tag names to tag
        classes.  filename is to identify where the template text came
        from.

        The the file identified by filename is never referenced to
        allow external code to implement caching schemes.  It also
        allows you to do things like this:

        t = Template(tags, "<cache>",
        """<al-for iter="i" expr="range(20)">
          <al-value expr="i.value">
        </al-for>""")
        '''
        TAG_TEXT = 0
        TAG_NAME = 1
        TAG_ATTRIBS = 2
        TAG_CLOSE = 3
        WHITE_ALL = 'all'
        WHITE_STRIP = 'strip'
        WHITE_INDENT = 'indent'
        WHITE_NEWLINE = 'newline'

        state = TAG_TEXT
        white_space = WHITE_STRIP
        self.content = Content()
        parent_stack = []
        parent = self.content
        line_num = 1
        for part in _re_tag.split(text):
            if state == TAG_TEXT:
                str = part
                if str:
                    if white_space == WHITE_STRIP:
                        if str[0] == '\n':
                            str = _re_leading.sub('', str)
                    elif white_space == WHITE_INDENT:
                        if str[0] == '\n':
                            str = str[1:]
                    elif white_space == WHITE_NEWLINE:
                        str = '\n'
                if str:
                    parent.append(Text(str))
                white_space = WHITE_STRIP
                state = TAG_NAME
            elif state == TAG_NAME:
                name = part.lower()
                if name[0] == '/':
                    # close tag - pop parent
                    name = name[1:]
                    if not parent_stack:
                        raise ApplicationError('%s:%s: "%s"; unexpected "%s" end tag' % (filename, line_num, part, name))
                    if parent.name != name:
                        raise ApplicationError('%s:%s: "%s"; expected "%s" end tag' % (filename, line_num, part, parent.name))
                    parent = parent_stack[-1]
                    del parent_stack[-1]
                    is_open = 0
                else:
                    # open tag - find class
                    tagclass = ctx.get_tagclass(name)
                    if not tagclass:
                        raise ApplicationError('%s:%s: undefined tag "%s"' % \
                                               (filename, line_num, name))
                    is_open = 1
                state = TAG_ATTRIBS
            elif state == TAG_ATTRIBS:
                bits = _re_attrib.split(part)
                dict = {}
                attrib_order = []
                offset = 0
                while 1:
                    attrbits = bits[offset: offset + 5]
                    if len(attrbits) < 5:
                        break
                    offset = offset + 5
                    white, name1, value1, value2, name2 = attrbits
                    if name1:
                        name = name1.lower()
                        if value1 is not None:
                            value = _re_dequote.sub(r'\1', value1)
                        else:
                            value = _re_dequote.sub(r'\1', value2)
                    else:
                        name = name2.lower()
                        value = None
                    if name == 'whitespace':
                        if value not in (WHITE_ALL, WHITE_STRIP, WHITE_INDENT,
                                         WHITE_NEWLINE, None):
                            raise ApplicationError('%s:%s: illegal whitespace value "%s"' % \
                                                   (filename, line_num, value))
                        white_space = value
                    else:
                        dict[name] = value
                        attrib_order.append(name)
                if is_open:
                    tag = tagclass(ctx, filename, line_num, dict)
                    tag.set_attrib_order(attrib_order)
                    parent.append(tag)
                    if tag.has_content():
                        parent_stack.append(parent)
                        parent = tag
                state = TAG_CLOSE
            elif state == TAG_CLOSE:
                if part.endswith('/>'):
                    if tag.has_content():
                        parent = parent_stack[-1]
                        del parent_stack[-1]
                else:
                    pass
                state = TAG_TEXT
            line_num = line_num + part.count('\n')
        if state not in (TAG_TEXT, TAG_NAME):
            raise ApplicationError('%s:%s: incomplete tag definition' % \
                                   (filename, line_num))
        if parent_stack:
            raise ApplicationError('%s:%s: missing close tag for "%s"' % \
                                   (filename, parent.line_num, parent.name))

    def __repr__(self):
        return 'Template(%s)' % self.content

    def to_html(self, ctx):
        '''Execute template within an execution context.
        '''
        self.content.to_html(ctx)


if __name__ == '__main__':
    import sys
    from albatross import context
    ctx = context.SimpleContext('../test')
    t = Template(ctx, '<ether>', '''
<al-lookup name="xlat">
 <al-item expr="0">Zero</al-item>
 <al-item expr="1">One</al-item>
 <al-item expr="2">Two</al-item>
 Many</al-lookup>
<al-for iter="i" expr="range(12)">
 <al-value expr="i.value()" lookup="xlat" whitespace>
</al-for>
''')
    t.to_html(ctx)
    ctx.flush_content()



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#

# Note: session_dir should not be a publically readable directory, or 
# local users will be able to steal sessions.

import os
import stat
import sys
import binascii
import random
import errno
import struct
import os.path

try:
    import Cookie
except ImportError:
    pass

from albatross.app import AppContext, Application, PageObjectMixin,\
     PageModuleMixin
from albatross.context import SessionBase, NameRecorderMixin,\
     PickleSignMixin, CachingTemplateLoaderMixin
from albatross.randompage import RandomPageModuleMixin
from albatross.common import *


# File based session recording
class SessionFileContextMixin(SessionBase):

    def __init__(self):
        SessionBase.__init__(self)
        self.__sesid = None

    def _get_sesid_from_cookie(self):
        hdr = self.request.get_header('Cookie')
        if hdr:
            c = Cookie.SimpleCookie(hdr)
            try:
                return os.path.basename(c[self.app.ses_appid()].value)
            except (OSError, KeyError):
                pass
        return None

    def sesid(self):
        return self.__sesid

    def load_session(self):
        sesid = self._get_sesid_from_cookie()
        text = None
        if sesid:
            text = self.app.get_session(sesid)
        if not text:
            sesid = self.app.new_session()
        self.__sesid = sesid
        if text:
            try:
                self.decode_session(text)
            except:
                self.app.del_session(sesid)
                raise
        c = Cookie.SimpleCookie()
        c[self.app.ses_appid()] = self.__sesid
        path = self.absolute_base_url()
        c[self.app.ses_appid()]['path'] = path
        prefix_len = len('Set-Cookie: ')
        self.set_header('Set-Cookie', str(c)[prefix_len:])

    def save_session(self):
        if self.should_save_session():
            text = self.encode_session()
            self.app.put_session(self.__sesid, text)

    def remove_session(self):
        SessionBase.remove_session(self)
        if self.__sesid is not None:
            self.app.del_session(self.__sesid)


class SessionFileAppMixin:

    def __init__(self, appid, session_dir):
        self.__appid = appid
        self.__session_dir = session_dir

    def ses_appid(self):
        return self.__appid

    def get_session(self, sesid):
        text = None
        try:
            text = open(self._ses_filename(sesid), "rb").read()
        except IOError, (eno, estr):
            if eno != errno.ENOENT:
                self.del_session(sesid)
                raise ServerError("read session file: %s: %s" %
                                  (self._ses_filename(sesid), estr))
        return text

    def new_session(self):
        while 1:
            try:
                text = open('/dev/urandom').read(8)
            except (OSError, IOError):
                text = struct.pack('d', random.random())
            sesid = binascii.hexlify(text)

            try:
                fd = os.open(self._ses_filename(sesid),
                             os.O_WRONLY | os.O_CREAT | os.O_EXCL, 0700)
            except OSError, (eno, estr):
                if eno == errno.ENOENT:
                    try:
                        os.makedirs(self.__session_dir, 0700)
                    except OSError, (eno, estr):
                        raise ServerError("create session dir: %s: %s" %
                                          (self.__session_dir, estr))
                    continue
                if eno != errno.EEXIST:
                    raise ServerError("new_session: %s: %s" %
                                      (self._ses_filename(sesid), estr))
                continue

            os.close(fd)
            return sesid

    def put_session(self, sesid, text):
        try:
            file = open(self._ses_filename(sesid), "wb")
            file.write(text)
            file.close()
        except IOError, (eno, estr):
            raise ServerError("write session file: %s: %s" %
                              (self._ses_filename(sesid), estr))

    def del_session(self, sesid):
        try:
            os.remove(self._ses_filename(sesid))
        except OSError:
            pass

    def _ses_filename(self, sesid):
        if os.path.dirname(sesid) != '':
            raise SecurityError("bad session id: %s" % (sesid))
        return os.path.join(self.__session_dir, sesid + '.als')


class SessionFileAppContext(AppContext,
                            NameRecorderMixin,
                            SessionFileContextMixin):

    def __init__(self, app):
        AppContext.__init__(self, app)
        NameRecorderMixin.__init__(self)
        SessionFileContextMixin.__init__(self)


class SimpleSessionFileApp(PickleSignMixin,
                            Application,
                            CachingTemplateLoaderMixin,
                            PageObjectMixin,
                            SessionFileAppMixin):

    def __init__(self, base_url, template_path, start_page, secret,
                 session_appid, session_dir):
        Application.__init__(self, base_url)
        PickleSignMixin.__init__(self, secret)
        CachingTemplateLoaderMixin.__init__(self, template_path)
        PageObjectMixin.__init__(self, start_page)
        SessionFileAppMixin.__init__(self, session_appid, session_dir)

    def create_context(self):
        return SessionFileAppContext(self)


class ModularSessionFileApp(PickleSignMixin,
                            Application,
                            CachingTemplateLoaderMixin,
                            PageModuleMixin,
                            SessionFileAppMixin):

    def __init__(self, base_url, module_path, template_path, start_page, secret,
                 session_appid, session_dir):
        Application.__init__(self, base_url)
        PickleSignMixin.__init__(self, secret)
        CachingTemplateLoaderMixin.__init__(self, template_path)
        PageModuleMixin.__init__(self, module_path, start_page)
        SessionFileAppMixin.__init__(self, session_appid, session_dir)

    def create_context(self):
        return SessionFileAppContext(self)


class RandomModularSessionFileApp(PickleSignMixin,
                                  Application,
                                  CachingTemplateLoaderMixin,
                                  RandomPageModuleMixin,
                                  SessionFileAppMixin):

    def __init__(self, base_url, page_path, start_page, secret,
                 session_appid, session_dir):
        Application.__init__(self, base_url)
        PickleSignMixin.__init__(self, secret)
        CachingTemplateLoaderMixin.__init__(self, page_path)
        RandomPageModuleMixin.__init__(self, page_path, start_page)
        SessionFileAppMixin.__init__(self, session_appid, session_dir)

    def create_context(self):
        return SessionFileAppContext(self)




#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# SessionServer functionality tests
#
# $Id: ses_server.py,v 1.8 2003/09/21 04:40:31 djc Exp $
#

import os, sys

try:
    base_dir = os.path.dirname(sys.modules[__name__].__file__)
except AttributeError:
    base_dir = ''
sys.path.insert(0, os.path.join(base_dir, '../../session-server'))

import unittest
import albatross

import request

class Vars:
    pass

class SessionServerApp(albatross.SessionServerAppMixin):
    def module_path(self):
        pass

class SessionServer(albatross.ResponseMixin, albatross.SessionServerContextMixin):
    """ 
    This corresponds to the object albatross users know as the ctx,
    although since we aren't using the normal set of mixins, we have to
    supply our own locals and request members.
    """
    def __init__(self, request):
        self.app = SessionServerApp('test')
        self.request = request
        self.locals = Vars()
        albatross.ResponseMixin.__init__(self)
        albatross.SessionServerContextMixin.__init__(self)

    def absolute_base_url(self):
        return ''

    def test_save(self):
        self.write_headers()
        self.save_session()

class Session:
    """
    This represents a virtual "user" - essentially just a place to store
    their session id while the ctx objects come and go.
    """
    def __init__(self):
        self.request = request.DummyCookieRequest()

    def new_context(self):
        return SessionServer(self.request)

class ServerCase(unittest.TestCase):
    def setUp(self):
        # We assume the user has started the session server - we don't
        # have a portable way to do this ourselves.
        pass

    def tearDown(self):
        pass

    def new_session(self):
        return Session()



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# Basic functionality tests - save and restore a single context
#
# $Id: concurrent.py,v 1.3 2002/07/30 07:33:51 andrewm Exp $
#

import unittest

class BasicCase(unittest.TestCase):
    def check_file(self):
        pass

    def check_server(self):
        pass

    def check_hiddenfield(self):
        pass

class BasicSuite(unittest.TestSuite):
    # add modules with tests here
    test_list = [
        "check_file",
        "check_server",
        "check_hiddenfield",
    ]

    def __init__(self):
        unittest.TestSuite.__init__(self, map(BasicCase, self.test_list))

def suite():
    return BasicSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='BasicSuite')



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# Test NameRecorderMixin functionality
#
# Unfortunately, we need to have intimate knowledge of the encoding scheme used
# by the NameRecorderMixin - changes there will need to be reflected here.
#
# $Id: recorder.py,v 1.6 2003/09/21 04:40:31 djc Exp $

import re, os, sys

import unittest
import base64, zlib, cPickle

import albatross

"""
Test field recording action

Set up all the appropriate infrastructure, then run a template that
contains an <al-form>. Check that the resulting html contains a __albform__
hidden field with an appropriate value.
"""
albform_re = re.compile(r'<input type="hidden" name="__albform__" value="([^"]*)" />', re.I|re.M|re.S)

class RecorderTestApp(albatross.Application):
    def __init__(self):
        albatross.Application.__init__(self, 'here')

    def pickle_sign(self, text):
        # albatross.Application discards text, we need a pass-thru
        return text

    def pickle_unsign(self, text):
        return text

class RecorderTestContext(albatross.TemplateLoaderMixin,
                  albatross.AppContext, albatross.NameRecorderMixin,
                  albatross.StubSessionMixin):
    def __init__(self, template_path):
        albatross.TemplateLoaderMixin.__init__(self, template_path)
        albatross.AppContext.__init__(self, RecorderTestApp())
        albatross.NameRecorderMixin.__init__(self)

    def current_url(self):
        return 'here'

class RecorderTestCase(unittest.TestCase):
    template_dir = "recorder"

    def runTest(self):
        base_dir = os.path.dirname(sys.modules[__name__].__file__)
        template_dir = os.path.join(base_dir, self.template_dir)
        self.ctx = RecorderTestContext(template_dir)

        self.ctx.push_content_trap()
        tmpl = self.ctx.load_template("recorder.html")
        tmpl.to_html(self.ctx)
        text = self.ctx.pop_content_trap()

        expect = {'this': 0, 'that': 1}
        result = None
        match = albform_re.search(text)
        self.failUnless(match, 
                        "__albform__ regular expression didn't find anything")
        text = base64.decodestring(match.group(1))
        text = zlib.decompress(text)
        result = cPickle.loads(text)
        self.assertEqual(result, expect)

"""
Test request merging action
"""
merge_data = [
    # var        list?  input           expect
    ('this',     0,     'one',          'one'),
    ('that',     1,     'two',          ['two']),
    ('another',  1,     ['three'],      ['three']),
    ('whatever', 1,     None,           []),
]

class MergeTestRequest:
    def __init__(self):
        self.expect = {}
        self.fields = {}
        self.islist = {}
        for var, islist, input, output in merge_data:
            self.islist[var] = islist
            self.fields[var] = input
            self.expect[var] = output

        text = cPickle.dumps(self.islist, 1)
        text = zlib.compress(text)
        text = base64.encodestring(text)
        self.fields['__albform__'] = text
        self.expect['__page__'] = None
        self.expect['__pages__'] = []

    def has_field(self, field):
        return self.fields.has_key(field)

    def field_value(self, field):
        return self.fields[field]

class MergeTestCase(unittest.TestCase):
    def runTest(self):
        ctx = RecorderTestContext('.')
        ctx.set_request(MergeTestRequest())
        ctx.merge_request()
        got = ctx.locals.__dict__
        self.assertEqual(got, ctx.request.expect)

"""
Test namespace "protection"
"""
class UnderscoreRequest:
    def __init__(self):
        self.__field = '_protected'

    def has_field(self, name):
        return name == self.__field

    def field_value(self, name):
        if name == self.__field:
            return self.__field
        else:
            raise KeyError

    def field_names(self):
        return [self.__field]

class UnderscoreTestCase(unittest.TestCase):
    def runTest(self):
        ctx = RecorderTestContext('.')
        ctx.set_request(UnderscoreRequest())
        self.failUnlessRaises(albatross.SecurityError, ctx.merge_request)

"""
Driver for the above tests
"""

class RecorderSuite(unittest.TestSuite):
    def __init__(self):
        unittest.TestSuite.__init__(self)
        self.addTest(RecorderTestCase())
        self.addTest(MergeTestCase())
        self.addTest(UnderscoreTestCase())

def suite():
    return RecorderSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



#!/usr/bin/python
import os
from albatross import SimpleContext, TemplateLoadError

script_name = os.environ['SCRIPT_NAME']
request_uri = os.environ['REQUEST_URI']
page = request_uri[len(script_name) + 1:]
if not page:
    page = 'main.html'

ctx = SimpleContext('templ')
ctx.load_template('macros.html').to_html(ctx)
ctx.locals.page = page
try:
    templ = ctx.load_template(page)
except TemplateLoadError:
    templ = ctx.load_template('oops.html')

templ.to_html(ctx)

print 'Content-Type: text/html'
print
ctx.flush_content()



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
from mod_python import util, apache
from albatross.cgiapp import RequestFields, FileField


class Request(RequestFields):

    def __init__(self, req):
        self.__req = req
        RequestFields.__init__(self, util.FieldStorage(self.__req))
        self.__status = 200

    def get_uri(self):
        return self.__req.uri

    def get_path_info(self):
        return self.__req.path_info

    def get_servername(self):
        try:
            return self.__req.hostname                          # mod_python 3
        except AttributeError:
            return self.__req.connection.server.server_hostname # mod_python 2

    def get_header(self, name):
        if self.__req.headers_in.has_key(name):
            return self.__req.headers_in[name]

    def write_header(self, name, value):
        if name.lower() == 'content-type':
            self.__req.content_type = value
        else:
            self.__req.headers_out[name] = value

    def end_headers(self):
        self.__req.send_http_header()

    def redirect(self, loc):
        self.write_header('Location', loc)
        return apache.HTTP_MOVED_PERMANENTLY

    def write_content(self, data):
        self.__req.write(data)

    def set_status(self, status):
        self.__status = status

    def status(self):
        if self.__status == 200:
            return 0
        return self.__status



from albatross import ModularSessionApp, SessionAppContext
from albatross.apacheapp import Request
import popviewlib


class AppContext(SessionAppContext):

    def open_mbox(self):
        if hasattr(self.locals, 'mbox'):
            return
        self.locals.mbox = popviewlib.Mbox(self.locals.username, self.locals.passwd)

    def read_msg(self):
        if hasattr(self.locals, 'msg'):
            return
        self.locals.msg = self.locals.mbox[int(self.locals.msgnum) - 1]
        self.locals.msg.read_body()


class App(ModularSessionApp):

    def __init__(self):
        ModularSessionApp.__init__(self,
                                   base_url='popview.py',
                                   module_path='-=-install_dir-=-',
                                   template_path='-=-install_dir-=-',
                                   start_page='login',
                                   secret='-=-secret-=-',
                                   session_appid='popview5')

    def create_context(self):
        return AppContext(self)


app = App()


def handler(req):
    return app.run(Request(req))



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Form functionality tests.
#
# $Id: form.py,v 1.18 2003/09/21 04:40:31 djc Exp $

import unittest
import albatross
import albatross_test
import time

class InputCase(albatross_test.AlbatrossTestCase):
    template_dir = "form"

    def check_input_text(self):
        self.ctx.locals.spam = "canned meat product"
        self.html_test("input-text.html", '<input name="spam" value="canned meat product" />')

    def check_input_text_escaping(self):
        self.ctx.locals.spam = '&<>"'
        self.html_test("input-text.html", '<input name="spam" value="&amp;&lt;&gt;&quot;" />')

    def check_input_text_zero(self):
        self.ctx.locals.spam = 0
        self.html_test("input-text.html", '<input name="spam" value="0" />')

    def check_input_text_expr(self):
        self.ctx.locals.str = "canned meat product"
        self.html_test("input-text-expr.html", '<input name="spam" value="canned meat product" />')

    def check_input_text_expr_escaping(self):
        self.ctx.locals.str = '&<>"\''
        self.html_test("input-text-expr.html", '<input name="spam" value="&amp;&lt;&gt;&quot;&#39;" />')

    def check_input_nameexpr_text(self):
        self.ctx.locals.str = 'spam'
        self.ctx.locals.num = 69
        self.ctx.locals.spam_69 = 'canned meat product'
        self.html_test("input-nameexpr-text.html", '<input type="text" name="spam_69" value="canned meat product" />')

    def check_input_password(self):
        self.html_test("input-password.html", '<input type="password" name="pwd" value="secret" />')

    def check_input_radio(self):
        self.ctx.locals.swallow = 'African'
        self.html_test("input-radio.html", """\
<input type="radio" name="swallow" value="African" checked />
<input type="radio" name="swallow" value="European" />
""")

    def check_input_radio_expr(self):
        self.ctx.locals.str = 'African'
        self.html_test("input-radio-expr.html", """\
<input type="radio" name="swallow" value="African" checked />
<input type="radio" name="swallow" value="European" />
""")

    def check_input_radio_valueexpr(self):
        self.ctx.locals.str = 'African'
        self.html_test("input-radio-valueexpr.html", """\
<input type="radio" name="swallow" value="African" checked />
""")

    def check_input_checkbox(self):
        self.ctx.locals.selected = ['two','four']
        self.ctx.locals.vexpr = 'four'
        self.html_test("input-checkbox.html", """\
<input type="checkbox" name="nums" value="one" />
<input type="checkbox" name="nums" value="two" checked />
<input type="checkbox" name="nums" value="three" />
<input type="checkbox" name="nums" value="four" checked />
""")

    def check_missing_multiple(self):
        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "missing-multiple.html")

    def check_illegal_multiple(self):
        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "illegal-multiple.html")

    def check_defined_single(self):
        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "defined-single.html")

    def check_defined_multiple(self):
        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, "defined-multiple.html")

    def check_missing_name(self):
        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, 
                          "missing-name.html")

    def check_null_name(self):
        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, 
                          "null-name.html")

class InputSuite(unittest.TestSuite):
    test_list = (
        "check_input_text",
        "check_input_text_escaping",
        "check_input_text_zero",
        "check_input_text_expr",
        "check_input_text_expr_escaping",
        "check_input_nameexpr_text",
        "check_input_password",
        "check_input_radio",
        "check_input_radio_expr",
        "check_input_radio_valueexpr",
        "check_input_checkbox",
        "check_missing_multiple",
        "check_illegal_multiple",
        "check_defined_single",
        "check_defined_multiple",
        "check_missing_name",
        "check_null_name",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(InputCase, self.test_list))


class SelectCase(albatross_test.AlbatrossTestCase):
    template_dir = "form"

    def check_select(self):
        self.ctx.locals.sel = 'spam'
        self.html_test("select.html", '<select name="sel"><option selected>spam</option><option>eggs</option></select>')

    def check_select_evaluate(self):
        self.ctx.locals.sel = 'spam'
        self.ctx.locals.val = 'spam'
        self.html_test("select-evaluate.html", '<select name="sel"><option selected>spam</option><option>eggs</option></select>')

    def check_select_value(self):
        self.ctx.locals.sel = 'spam'
        self.html_test("select-value.html", '<select name="sel"><option value="spam" selected>canned meat product</option><option>eggs</option></select>')

    def check_select_optionexpr(self):
        self.ctx.locals.sel = '3'
        self.ctx.locals.opt = range(5)
        self.html_test("select-optionexpr.html", """\
<select name="sel"><option>0</option>
<option>1</option>
<option>2</option>
<option selected>3</option>
<option>4</option>
</select>""")

    def check_select_optionexpr_pairs(self):
        self.ctx.locals.sel = '2'
        self.ctx.locals.pairs = [(1, 'Spam'), (2, 'Eggs'), (3, 'Bacon')]
        self.html_test("select-optionexpr-pairs.html", """\
<select name="sel"><option value="1">Spam</option>
<option selected value="2">Eggs</option>
<option value="3">Bacon</option>
</select>""")

    def check_select_optionexpr_escaping(self):
        self.ctx.locals.sel = '<'
        self.ctx.locals.opt = ['&', '<', '>', '"']
        self.html_test("select-optionexpr.html", """\
<select name="sel"><option>&amp;</option>
<option selected>&lt;</option>
<option>&gt;</option>
<option>&quot;</option>
</select>""")

    def check_select_optionexpr_pair_escaping(self):
        self.ctx.locals.sel = '<'
        self.ctx.locals.pairs = [('&', '&'), ('<', '<'), ('>', '>'), ('"', '"')]
        self.html_test("select-optionexpr-pairs.html", """\
<select name="sel"><option value="&amp;">&amp;</option>
<option selected value="&lt;">&lt;</option>
<option value="&gt;">&gt;</option>
<option value="&quot;">&quot;</option>
</select>""")

class SelectSuite(unittest.TestSuite):
    test_list = (
        "check_select",
        "check_select_evaluate",
        "check_select_value",
        "check_select_optionexpr",
        "check_select_optionexpr_pairs",
        "check_select_optionexpr_escaping",
        "check_select_optionexpr_pair_escaping",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(SelectCase, self.test_list))


class TextareaCase(albatross_test.AlbatrossTestCase):
    template_dir = "form"

    def check_textarea(self):
        self.html_test("textarea.html", '<textarea name="msg">content\n</textarea>')

    def check_textarea_fromapp(self):
        self.ctx.locals.msg = "content\n"
        self.html_test("textarea-fromapp.html", '<textarea name="msg">content\n</textarea>')

    def check_textarea_fromapp_escaping(self):
        self.ctx.locals.msg = '&<>"'
        self.html_test("textarea-fromapp.html", '<textarea name="msg">&amp;&lt;&gt;&quot;</textarea>')


class TextareaSuite(unittest.TestSuite):
    test_list = (
        "check_textarea",
        "check_textarea_fromapp",
        "check_textarea_fromapp_escaping",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(TextareaCase, self.test_list))


class FormCase(albatross_test.AlbatrossTestCase):
    template_dir = "form"

    def check_nested_form(self):
        self.assertRaises(albatross.ApplicationError, self.gen_html, self.ctx, 
                          "nested-form.html")


class FormSuite(unittest.TestSuite):
    test_list = (
        'check_nested_form',
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(FormCase, self.test_list))
        self.addTest(InputSuite())
        self.addTest(SelectSuite())
        self.addTest(TextareaSuite())

def suite():
    return FormSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



import os
import sys
import string
import formatter
import imp
import struct
import random
import binascii
import re

class Error(Exception):
    pass

def random_secret():
    try:
        text = open('/dev/urandom').read(16)
    except:
        text = struct.pack('dd', random.random(), random.random())
    return binascii.hexlify(text)

def make_directory(name):
    parent, child = os.path.split(name)
    if child:
        make_directory(parent)
    try:
        os.stat(name)[0]
        return
    except (IOError, OSError):
        pass
    print 'creating directory %s' % name
    os.mkdir(name)

class File:
    def __init__(self, name, dest):
        self.name = name
        self.dest = dest

    def dest_pathname(self):
        if self.dest[-1] == os.sep:
            dirname, name = os.path.split(self.name)
            return os.path.join(self.dest, name)
        return self.dest

    def dest_dirname(self):
        dirname, name = os.path.split(self.dest_pathname())
        return dirname

    def copyfile(self, installer):
        data = open(self.name).read()
        if re.search(r'-=-secret-=-', data):
            secret = installer.get_secret()
            data = re.sub(r'-=-secret-=-', str(secret), data)
        data = re.sub(r'-=-install_dir-=-', self.dest_dirname(), data)

        dest = self.dest_pathname()
        open(dest, 'w').write(data)
        return dest

class ExecFile(File):
    def install(self, installer):
        dest = self.copyfile(installer)
        print 'copy exec %s to %s' % (self.name, dest)
        if os.name == 'posix':
            os.chmod(dest, 0755)

class ModuleFile(File):
    def install(self, installer):
        dest = self.copyfile(installer)
        print 'copy module %s to %s' % (self.name, dest)

class HtmlFile(File):
    def install(self, installer):
        dest = self.copyfile(installer)
        print 'copy html %s to %s' % (self.name, dest)

class DataFile(File):
    def install(self, installer):
        dest = self.copyfile(installer)
        print 'copy data %s to %s' % (self.name, dest)

class ImageFile(File):
    def install(self, installer):
        dest = self.copyfile(installer)
        print 'copy image %s to %s' % (self.name, dest)

class Config:
    def __init__(self):
        self.__has_changed = 0

    def __str__(self):
        parts = []
        for name in dir(self):
            if type(getattr(self, name)) is not type(''):
                continue
            parts.append('%s = "%s"' % (name, getattr(self, name)))
        parts.append('')
        return string.join(parts, '\n')

    def set_value(self, attr, value):
        if not hasattr(self, attr) or getattr(self, attr) != value:
            self.__has_changed = 1
            setattr(self, attr, value)

    def has_changed(self):
        return self.__has_changed

class Installer:
    def __init__(self, name, cgi = 0, apache = 0, doc = 0):
        msg = 'Installing %s' % name
        print msg
        print '=' * len(msg)
        self._name = name
        self._files = []
        self.load_config()
        if cgi:
            cgibin_dir = self.get_config('cgibin_dir')
            self.destdir = os.path.join(cgibin_dir, self._name)
        elif apache:
            modpython_dir = self.get_config('modpython_dir')
            self.destdir = os.path.join(modpython_dir, self._name)
        elif doc:
            doc_dir = self.get_config('doc_dir')
            self.destdir = os.path.join(doc_dir, self._name)
        else:
            raise Error('must specify either cgi = 1 or apache = 1 or doc = 1')

    def __del__(self):
        self.save_config()

    def get_config(self, name):
        if hasattr(self.config, name):
            return getattr(self.config, name)
        w = formatter.DumbWriter()
        f = formatter.AbstractFormatter(w)
        f.add_flowing_data('Please enter a value for ')
        f.add_flowing_data(name)
        f.add_flowing_data(' which is ')
        if name[-7:] == '_secret':
            f.add_flowing_data('the secret key used to sign pickles' \
                               ' sent to the browser from the ')
            f.add_flowing_data(self._name)
            f.add_flowing_data(' program.')
            default = '<random key>'
        elif name == 'cgibin_dir':
            f.add_flowing_data('the top level directory where CGI Albatross')
            f.add_flowing_data(' samples will be installed.')
            default = '/usr/lib/cgi-bin'
        elif name == 'modpython_dir':
            f.add_flowing_data('the top level directory where mod_python')
            f.add_flowing_data(' Albatross samples will be installed.')
            default = '/var/www'
        elif name == 'doc_dir':
            f.add_flowing_data('the top level directory where ')
            f.add_flowing_data(' Albatross html documents will be installed.')
            default = '/var/www'
        f.end_paragraph(1)
        value = raw_input('Enter %s [%s]: ' % (name, default))
        if not value:
            value = default
        self.config.set_value(name, value)
        return value

    def load_config(self):
        pathname = sys.modules['install'].__file__
        dirname, name = os.path.split(pathname)
        self.cfgname = os.path.join(dirname, 'install.cfg')
        self.config = Config()
        try:
            execfile(self.cfgname, globals(), self.config.__dict__)
        except IOError:
            pass

    def save_config(self):
        if self.config.has_changed():
            open(self.cfgname, 'w').write('%s' % self.config)

    def get_secret(self):
        dirname, name = os.path.split(self._name)
        attr = '%s_secret' % name
        secret = self.get_config(attr)
        if secret == '<random key>':
            self.config.set_value(attr, random_secret())
        return getattr(self.config, attr)

    def add_exec(self, name, dest = ''):
        dest = os.path.join(self.destdir, dest)
        self._files.append(ExecFile(name, dest))

    def add_module(self, name, dest = ''):
        dest = os.path.join(self.destdir, dest)
        self._files.append(ModuleFile(name, dest))

    def add_html(self, name, dest = ''):
        dest = os.path.join(self.destdir, dest)
        self._files.append(HtmlFile(name, dest))

    def add_data(self, name, dest = ''):
        dest = os.path.join(self.destdir, dest)
        self._files.append(DataFile(name, dest))

    def add_image(self, name, dest = ''):
        dest = os.path.join(self.destdir, dest)
        self._files.append(ImageFile(name, dest))

    def install(self):
        # Make sure all directories exist
        dirnames = {}
        for file in self._files:
            dirnames[file.dest_dirname()] = 1
        dirnames = dirnames.keys()
        dirnames.sort()
        for name in dirnames:
            make_directory(name)
        # Install files
        for file in self._files:
            file.install(self)
        print

class Catalog:
    def __init__(self):
        self.dirnames = []

    def append(self, name):
        self.dirnames.append(name)

    def install_all(self):
        self.dirnames.sort()
        for dirname in self.dirnames:
            if not self.install(dirname):
                break

    def install(self, dirname):
        cwd = os.getcwd()
        os.chdir(dirname)
        result = os.system('python install.py')
        os.chdir(cwd)
        return result == 0

# Walk the samples directory finding information about samples
def visit_dir(catalog, dirname, names):
    if dirname != rootdir and 'install.py' in names:
        catalog.append(dirname)

if __name__ == '__main__':
    # Find the root of the samples directory
    rootdir, basename = os.path.split(sys.argv[0])
    if not rootdir:
        rootdir = '.'
    catalog = Catalog()
    os.path.walk(rootdir, visit_dir, catalog)
    catalog.install_all()



#!/usr/bin/python
import os
from albatross import SimpleContext

ctx = SimpleContext('.')
templ = ctx.load_template('simple.html')

keys = os.environ.keys()
keys.sort()
ctx.locals.keys = keys
ctx.locals.environ = os.environ

templ.to_html(ctx)

print 'Content-Type: text/html'
print
ctx.flush_content()



import sys
sys.path.insert(0, '../..')
import install
from install import Installer

i = Installer('alsamp/content1', cgi = 1)
i.add_exec('content.py')
i.add_html('main.html', 'templ/')
i.add_html('oops.html', 'templ/')
i.add_html('other.html', 'templ/')
i.install()



#!/usr/bin/python
import cgi
from albatross import SimpleContext

ctx = SimpleContext('.')
ctx.locals.form = cgi.FieldStorage()

templ = ctx.load_template('form.html')
templ.to_html(ctx)

print 'Content-Type: text/html'
print
ctx.flush_content()



#!/usr/bin/python
from albatross import SimpleSessionApp, SessionAppContext
from albatross.cgiapp import Request
import popviewlib


class LoginPage:

    name = 'login'

    def page_process(self, ctx):
        if ctx.req_equals('login'):
            if ctx.locals.username and ctx.locals.passwd:
                try:
                    ctx.open_mbox()
                    ctx.add_session_vars('username', 'passwd')
                except poplib.error_proto:
                    return
                ctx.set_page('list')

    def page_display(self, ctx):
        ctx.run_template('login.html')


class ListPage:

    name = 'list'

    def page_process(self, ctx):
        if ctx.req_equals('msgnum'):
            ctx.set_page('detail')

    def page_display(self, ctx):
        ctx.open_mbox()
        ctx.run_template('list.html')


class DetailPage:

    name = 'detail'

    def page_process(self, ctx):
        if ctx.req_equals('list'):
            ctx.set_page('list')

    def page_display(self, ctx):
        ctx.open_mbox()
        ctx.read_msg()
        ctx.run_template('detail.html')


class AppContext(SessionAppContext):

    def open_mbox(self):
        if hasattr(self.locals, 'mbox'):
            return
        self.locals.mbox = popviewlib.Mbox(self.locals.username, self.locals.passwd)

    def read_msg(self):
        if hasattr(self.locals, 'msg'):
            return
        self.locals.msg = self.locals.mbox[int(self.locals.msgnum) - 1]
        self.locals.msg.read_body()


class App(SimpleSessionApp):

    def __init__(self):
        SimpleSessionApp.__init__(self,
                                  base_url='popview.py',
                                  template_path='.',
                                  start_page='login',
                                  secret='-=-secret-=-',
                                  session_appid='popview3')
        for page_class in (LoginPage, ListPage, DetailPage):
            self.register_page(page_class.name, page_class())

    def create_context(self):
        return AppContext(self)


app = App()


if __name__ == '__main__':
    app.run(Request())



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Test environment settings.
#
# $Id: env.py,v 1.5 2002/07/30 07:33:51 andrewm Exp $

import unittest
import albatross
import albatross_test

import os

class EnvCase(albatross_test.AlbatrossTestCase):
    template_dir = "env"

    # overload html_test() to drop environment into local context
    def html_test(self, template_file, expect):
        self.ctx.locals.env = os.environ
        albatross_test.AlbatrossTestCase.html_test(self, template_file, expect)

    def check_home(self):
        self.html_test("home.html", os.environ['HOME'])


class EnvSuite(unittest.TestSuite):
    test_list = (
        "check_home",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(EnvCase, self.test_list))

def suite():
    return EnvSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Test tree constructs.
#
# $Id: tree.py,v 1.7 2003/09/21 04:40:31 djc Exp $

import unittest
import albatross
import albatross_test

one_we_prepared_earlier="""\
-a
 |-b
 | |-c
 | \-d
 \-e
   |-f
   | \-g
   |   |-h
   |   \-i
   |-j
   \-k
     |-l
     \-m
"""

tree_input_folded="""\
<form action="here">
 <table>
  <tr>
    <td width="16">
      <input type="image" src="close.png" width="13" height="13" border="0" name="treefold,n,a" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,a" value="on" /></td>
    <td colspan="999" nowrap>a</td>
   </tr>
  </table>
<input type="hidden" name="__albform__" value="eJwDAAAAAAE=
" />
</form>"""

tree_input_unfolded="""\
<form action="here">
 <table>
  <tr>
    <td width="16">
      <input type="image" src="open.png" width="13" height="13" border="0" name="treefold,n,a" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,a" value="on" /></td>
    <td colspan="999" nowrap>a</td>
   </tr>
  <tr>
    <td width="16" colspan="1" nowrap></td>
    <td width="16">
      <input type="image" src="close.png" width="13" height="13" border="0" name="treefold,n,b" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,b" value="on" /></td>
    <td colspan="999" nowrap>b</td>
   </tr>
  <tr>
    <td width="16" colspan="1" nowrap></td>
    <td width="16">
      <input type="image" src="open.png" width="13" height="13" border="0" name="treefold,n,e" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,e" value="on" /></td>
    <td colspan="999" nowrap>e</td>
   </tr>
  <tr>
    <td width="16" colspan="2" nowrap></td>
    <td width="16">
      <input type="image" src="close.png" width="13" height="13" border="0" name="treefold,n,f" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,f" value="on" /></td>
    <td colspan="999" nowrap>f</td>
   </tr>
  <tr>
    <td width="16" colspan="2" nowrap></td>
    <td width="16" nowrap></td>
    <td width="16"><input type="checkbox" name="treeselect,n,j" value="on" /></td>
    <td colspan="999" nowrap>j</td>
   </tr>
  <tr>
    <td width="16" colspan="2" nowrap></td>
    <td width="16">
      <input type="image" src="close.png" width="13" height="13" border="0" name="treefold,n,k" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,k" value="on" /></td>
    <td colspan="999" nowrap>k</td>
   </tr>
  </table>
<input type="hidden" name="__albform__" value="eJwDAAAAAAE=
" />
</form>"""

tree_input_unfolded_selected="""\
<form action="here">
 <table>
  <tr>
    <td width="16">
      <input type="image" src="open.png" width="13" height="13" border="0" name="treefold,n,a" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,a" value="on" /></td>
    <td colspan="999" nowrap>a</td>
   </tr>
  <tr>
    <td width="16" colspan="1" nowrap></td>
    <td width="16">
      <input type="image" src="close.png" width="13" height="13" border="0" name="treefold,n,b" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,b" value="on" checked /></td>
    <td colspan="999" nowrap>b</td>
   </tr>
  <tr>
    <td width="16" colspan="1" nowrap></td>
    <td width="16">
      <input type="image" src="open.png" width="13" height="13" border="0" name="treefold,n,e" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,e" value="on" /></td>
    <td colspan="999" nowrap>e</td>
   </tr>
  <tr>
    <td width="16" colspan="2" nowrap></td>
    <td width="16">
      <input type="image" src="close.png" width="13" height="13" border="0" name="treefold,n,f" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,f" value="on" /></td>
    <td colspan="999" nowrap>f</td>
   </tr>
  <tr>
    <td width="16" colspan="2" nowrap></td>
    <td width="16" nowrap></td>
    <td width="16"><input type="checkbox" name="treeselect,n,j" value="on" checked /></td>
    <td colspan="999" nowrap>j</td>
   </tr>
  <tr>
    <td width="16" colspan="2" nowrap></td>
    <td width="16">
      <input type="image" src="close.png" width="13" height="13" border="0" name="treefold,n,k" /></td>
    <td width="16"><input type="checkbox" name="treeselect,n,k" value="on" /></td>
    <td colspan="999" nowrap>k</td>
   </tr>
  </table>
<input type="hidden" name="__albform__" value="eJwDAAAAAAE=
" />
</form>"""

class SimpleNode:
    def __init__(self, name, children = None):
        self.name = name
        if children is not None:
            self.children = children

class LazyNode:
    def __init__(self, name, children = None):
        self.name = name
        if children is not None:
            self.unloaded_children = children
            self.children = []
        self.children_loaded = 0

    def load_children(self,ctx):
        self.children = self.unloaded_children

    def albatross_alias(self):
        return self.name

def build_tree(Ctor):
    return Ctor('a', [Ctor('b', [Ctor('c'),
                                 Ctor('d')]),
                      Ctor('e', [Ctor('f', [Ctor('g', [Ctor('h'),
                                                       Ctor('i')])]),
                                 Ctor('j'),
                                 Ctor('k', [Ctor('l'),
                                            Ctor('m')])])])

class TreeCase(albatross_test.AlbatrossTestCase):
    template_dir = "tree"

    def check_tree(self):
        self.ctx.locals.tree = build_tree(SimpleNode)
        self.html_test("tree.html", one_we_prepared_earlier)

    def check_tree_lazy(self):
        self.ctx.locals.tree = build_tree(LazyNode)
        self.html_test("tree-lazy.html", "-a\n")
        self.ctx.locals.n.set_backdoor('treefold','a',1)
        self.html_test("tree-lazy.html", "-a\n |-b\n \-e\n")

    def check_tree_input(self):
        self.ctx.locals.tree = build_tree(LazyNode)
        self.html_test("tree-input.html", tree_input_folded)
        self.ctx.locals.n.set_backdoor('treefold','a',1)
        self.ctx.locals.n.set_backdoor('treefold','e',1)
        self.html_test("tree-input.html", tree_input_unfolded)
        self.ctx.locals.n.set_backdoor('treeselect','b',1)
        self.ctx.locals.n.set_backdoor('treeselect','j',1)
        self.html_test("tree-input.html", tree_input_unfolded_selected)


class TreeSuite(unittest.TestSuite):
    test_list = (
        "check_tree",
        "check_tree_lazy",
        "check_tree_input",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(TreeCase, self.test_list))

def suite():
    return TreeSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



import Sybase
from albatross import SimpleSessionApp
from albatross.apacheapp import Request


def get_table_list(ctx):
    db = ctx.app.db
    db.execute('use %s' % ctx.locals.database)
    c = db.cursor()
    c.execute('select name from sysobjects'
              ' where type in ("U", "S")'
              ' order by name')
    tables = map(lambda r: r[0], c.fetchall())
    ctx.locals.tables = tables


def get_database_list(ctx):
    c = ctx.app.db.cursor()
    c.execute('select name from master..sysdatabases')
    databases = map(lambda r: r[0], c.fetchall())
    ctx.locals.databases = databases


def get_table_desc(ctx):
    db = ctx.app.db
    db.execute('use %s' % ctx.locals.database)
    c = db.cursor()
    c.execute('select id from sysobjects'
              ' where type in ("U", "S") and name = "%s"' % ctx.locals.table)
    table_id, = c.fetchone()
    c.execute('select c.name, t.name, c.length, c.status'
              ' from syscolumns c, systypes t'
              ' where c.id = %d and c.usertype = t.usertype'
              ' order by c.colid' % table_id)
    ctx.locals.columns = c.fetchall()


def get_table_view(ctx):
    db = ctx.app.db
    db.execute('use %s' % ctx.locals.database)
    c = db.cursor()
    c.execute('select u.name from sysusers u, sysobjects o'
              ' where o.type in ("U","S") and o.name = "%s"'
              ' and o.uid = u.uid' % ctx.locals.table)
    owner, = c.fetchone()
    table = '%s.%s' % (owner, ctx.locals.table)
    c = db.cursor()
    c.execute('select * from %s' % table)
    ctx.locals.desc = c.description
    ctx.locals.rows = c.fetchall()


def request_field(ctx, name):
    if ctx.request.has_field(name):
        return ctx.request.field_value(name)


class TableList:

    name = 'table_list'

    def page_display(self, ctx):
        get_database_list(ctx)
        get_table_list(ctx)
        ctx.run_template('table-list.html')

    def page_process(self, ctx):
        if ctx.req_equals('changedb'):
            ctx.locals.database = request_field(ctx, 'database')
        elif ctx.req_equals('desc'):
            ctx.locals.table = request_field(ctx, 'desc')
            ctx.set_page('table_desc')
        elif ctx.req_equals('view'):
            ctx.locals.table = request_field(ctx, 'view')
            ctx.set_page('table_view')


class TableDesc:

    name = 'table_desc'

    def page_display(self, ctx):
        get_table_desc(ctx)
        ctx.run_template('table-desc.html')

    def page_process(self, ctx):
        if ctx.req_equals('changetable'):
            ctx.locals.table = request_field(ctx, 'table')
        elif ctx.req_equals('back'):
            ctx.set_page('table_list')


class TableView:

    name = 'table_view'

    def page_display(self, ctx):
        get_table_view(ctx)
        ctx.run_template('table-view.html')

    def page_process(self, ctx):
        if ctx.req_equals('changetable'):
            try:
                del ctx.locals.r
            except AttributeError:
                pass
            ctx.locals.table = request_field(ctx, 'table')
        elif ctx.req_equals('back'):
            ctx.set_page('table_list')


class App(SimpleSessionApp):

    def __init__(self):
        SimpleSessionApp.__init__(self,
                                  base_url='sybase.py',
                                  template_path='-=-install_dir-=-',
                                  start_page='table_list',
                                  secret='-=-secret-=-',
                                  session_appid='sybase')
        for page_class in (TableList, TableDesc, TableView):
            self.register_page(page_class.name, page_class())
        self.db = Sybase.connect('rat', 'sa', '')

    def load_session(self, ctx):
        SimpleSessionApp.load_session(self, ctx)
        ctx.default_session_var('database', 'master')
        ctx.default_session_var('tables', [])
        ctx.default_session_var('table', None)

    def create_context(self):
        ctx = SimpleSessionApp.create_context(self)
        ctx.run_template_once('macros.html')
        return ctx


app = App()


def handler(req):
    return app.run(Request(req))



#!/usr/bin/python
import time
from albatross import SimpleContext

class SlowProcess:
    def __getitem__(self, i):
        time.sleep(1)
        if i < 10:
            return i
        raise IndexError

ctx = SimpleContext('.')
templ = ctx.load_template('stream.html')
ctx.locals.process = SlowProcess()

print 'Content-Type: text/html'
print
templ.to_html(ctx)
ctx.flush_content()



import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/images', doc = 1)
i.add_image('open.png')
i.add_image('close.png')
i.add_image('ellipsis.png')
i.install()



import sys
sys.path.insert(0, '..')
import install
from install import Installer

i = Installer('alsamp/mpperf', apache = 1)
i.add_exec('mpperf.py')
i.add_module('main.py')
i.add_html('main.html')
i.add_data('htaccess', '.htaccess')
i.install()



#!/usr/bin/python
from albatross import SimpleApp
from albatross.cgiapp import Request


class Form:

    def page_enter(self, ctx):
        ctx.locals.text = ctx.locals.singleton = ctx.locals.group = \
                          ctx.locals.radio = ctx.locals.select = None
        ctx.locals.num = 0
        ctx.add_session_vars('num')

    def page_display(self, ctx):
        ctx.locals.num += 1
        ctx.run_template('form.html')


app = SimpleApp(base_url='form.py',
                template_path='.',
                start_page='form',
                secret='-=-secret-=-')
app.register_page('form', Form())


if __name__ == '__main__':
    app.run(Request())



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# Build and test all listed modules.
#
# The module files must have a function "suite" which returns
# an instance of unittest.TestSuite or unittest.TestCase.
#
# $Id: __init__.py,v 1.4 2002/07/30 07:33:51 andrewm Exp $

import unittest

class AllTestSuite(unittest.TestSuite):
    # add modules with tests here
    all_tests = [
        "basic",
        "concurrent",
    ]
    def __init__(self):
        unittest.TestSuite.__init__(self)
        for module_name in self.all_tests:
            module = __import__(module_name, globals())
            self.addTest(module.suite())

def suite():
    return AllTestSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='AllTestSuite')



import sys
import albatross


class Request:

    def get_uri(self):
        return 'http://www.com/fakeapp.py'

    def write_header(self, name, value):
        pass

    def end_headers(self):
        pass

    def write_content(self, data):
        sys.stdout.write(data)


app = albatross.SimpleApp(base_url='fakeapp.py',
                          template_path='.',
                          start_page='start',
                          secret='secret')

ctx = albatross.SessionAppContext(app)
ctx.set_request(Request())



import poplib


def page_process(ctx):
    if ctx.req_equals('login'):
        if ctx.locals.username and ctx.locals.passwd:
            try:
                ctx.open_mbox()
                ctx.add_session_vars('username', 'passwd')
            except poplib.error_proto:
                return
            ctx.set_page('list')


def page_display(ctx):
    ctx.run_template('login.html')




#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Test iterator constructs.
#
# $Id: iterator.py,v 1.10 2003/01/24 11:34:31 andrewm Exp $

import unittest
import albatross
import albatross_test

class IteratorCase(albatross_test.AlbatrossTestCase):
    template_dir = "iterator"

    def check_iterator(self):
        self.html_test("iterator.html", "01234")

    def check_nested(self):
        self.html_test("nested.html", "000102101112")

    def check_unclosed(self):
        self.assertRaises(albatross.ApplicationError, self.html_test, "unclosed.html", "(none)")

    def check_colsdown(self):
        self.html_test("down.html", "+035+146+2+")

    def check_colsacross(self):
        self.html_test("across.html", "+012+345+6+")

class IteratorSuite(unittest.TestSuite):
    test_list = (
        "check_iterator",
        "check_nested",
        "check_unclosed",
        "check_colsdown",
        "check_colsacross"
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(IteratorCase, self.test_list))

def suite():
    return IteratorSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# SessionFile functionality tests
#
# $Id: ses_file.py,v 1.7 2003/09/21 04:40:31 djc Exp $
#

import os, sys, errno

import unittest
import albatross

import request

class Vars:
    pass

class SessionFileApp(albatross.SessionFileAppMixin):
    def module_path(self):
        pass

class SessionFile(albatross.ResponseMixin, albatross.SessionFileContextMixin):
    """ 
    This corresponds to the object albatross users know as the ctx,
    although since we aren't using the normal set of mixins, we have to
    supply our own locals and request members.
    """
    def __init__(self, request, test_dir):
        self.app = SessionFileApp('test', test_dir)
        self.request = request
        self.locals = Vars()
        albatross.ResponseMixin.__init__(self)
        albatross.SessionFileContextMixin.__init__(self)

    def absolute_base_url(self):
        return ''

    def test_save(self):
        self.write_headers()
        self.save_session()

class Session:
    """
    This represents a virtual "user" - essentially just a place to store
    their session id while the ctx objects come and go.
    """
    def __init__(self, test_dir):
        self.test_dir = test_dir
        self.request = request.DummyCookieRequest()

    def new_context(self):
        return SessionFile(self.request, self.test_dir)

class FileCase(unittest.TestCase):
    def setUp(self):
        # This is because we may not be imported from the current directory
        try:
            base_dir = os.path.dirname(sys.modules[__name__].__file__)
        except AttributeError:
            base_dir = ''
        self.test_dir = os.path.join(base_dir, "sessionfiletmp")
        try:
            os.mkdir(self.test_dir)
        except OSError, (eno, estr):
            if eno == errno.EEXIST:
                self.cleandir(self.test_dir)
            else:
                raise OSError, (eno, estr)

    def tearDown(self):
        self.cleandir(self.test_dir)
        os.rmdir(self.test_dir)

    def cleandir(self, test_dir):
        for file in os.listdir(test_dir):
            if file[-4:] == '.als':             # Safety
                os.unlink(os.path.join(test_dir, file))

    def new_session(self):
        return Session(self.test_dir)



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# Application support which all application types inherit from

import os
import sys
import stat
import imp
import urlparse

from albatross.context import NamespaceMixin, ExecuteMixin, ResourceMixin,\
     NameRecorderMixin, HiddenFieldSessionMixin, PickleSignMixin,\
     CachingTemplateLoaderMixin, _caller_globals

from albatross.session import SessionServerContextMixin,\
     SessionServerAppMixin

from albatross import tags

from albatross.common import *


class Redirect:

    def __init__(self, loc):
        self.loc = loc

class ResponseMixin:

    '''Maintain an ordered dictionary of headers.
    '''

    def __init__(self):
        self.__sent_headers = 0
        self.__seqnum = 0
        self.__headers = {}
        self.set_header('Pragma', 'no-cache')
        self.set_header('Content-Type', 'text/html')

    def get_header(self, name):
        try:
            seqnum, value = self.__headers[name]
            return value
        except KeyError:
            pass

    def set_header(self, name, value):
        if self.__sent_headers:
            raise ApplicationError('headers have already been sent')
        try:
            seqnum, old_value = self.__headers[name]
            self.__headers[name] = (seqnum, value)
            return
        except KeyError:
            pass
        self.__headers[name] = (self.__seqnum, value)
        self.__seqnum = self.__seqnum + 1

    def del_header(self, name):
        if self.__sent_headers:
            raise ApplicationError('headers have already been sent')
        try:
            del self.__headers[name]
        except KeyError:
            pass

    def write_headers(self):
        if self.__sent_headers:
            raise ApplicationError('headers have already been sent')
        headers = []
        for name, seqnum_value in self.__headers.items():
            seqnum, value = seqnum_value
            headers.append((seqnum, name, value))
        headers.sort()
        for seqnum, name, value in headers:
            self.request.write_header(name, value)
        self.request.end_headers()
        self.__sent_headers = 1

    def send_content(self, data):
        if not self.__sent_headers:
            self.write_headers()
        self.request.write_content(data)

    def send_redirect(self, loc):
        cookie = self.get_header('Set-Cookie')
        if cookie:
            self.request.write_header('Set-Cookie', cookie)
        return self.request.redirect(loc)


# With an application object the registration of resources is managed
# by the application.
class AppContext(NamespaceMixin, ResponseMixin, ExecuteMixin):

    def __init__(self, app):
        NamespaceMixin.__init__(self)
        ResponseMixin.__init__(self)
        ExecuteMixin.__init__(self)
        self.locals.__page__ = None
        self.locals.__pages__ = []
        self.app = app

    # ----------------------------------------------------
    # Serve all requests for resources via the application

    def get_macro(self, name):
        return self.app.get_macro(name)

    def register_macro(self, name, macro):
        return self.app.register_macro(name, macro)

    def get_lookup(self, name):
        return self.app.get_lookup(name)

    def register_lookup(self, name, lookup):
        return self.app.register_lookup(name, lookup)

    def get_tagclass(self, name):
        return self.app.get_tagclass(name)

    # ----------------------------------------------------
    # Serve all requests for templates via the application

    def load_template(self, name):
        return self.app.load_template(name)

    def load_template_once(self, name):
        return self.app.load_template_once(name)

    def run_template(self, name):
        templ = self.app.load_template(name)
        self.set_globals(_caller_globals('run_template'))
        templ.to_html(self)

    def run_template_once(self, name):
        templ = self.app.load_template_once(name)
        if templ:
            self.set_globals(_caller_globals('run_template_once'))
            templ.to_html(self)

    # ----------------------------------------------------
    # Introduce some application specific methods
    def clear_locals(self):
        page, pages = self.locals.__page__, self.locals.__pages__
        NamespaceMixin.clear_locals(self)
        self.locals.__page__, self.locals.__pages__ = page, pages

    def __enter_page(self, name, args):
        self.locals.__page__ = name
        self.app.load_page(self)
        self.app.page_enter(self, args)

    def __leave_page(self, new_page_name):
        if self.locals.__page__ \
           and self.locals.__page__ != new_page_name:
            self.app.load_page(self)
            self.app.page_leave(self)

    def set_page(self, name, *args):
        self.__leave_page(name)
        self.__enter_page(name, args)

    def push_page(self, name, *args):
        if self.locals.__page__ \
           and self.locals.__page__ != name:
            self.locals.__pages__.append(self.locals.__page__)
        self.__enter_page(name, args)

    def pop_page(self, target_page = None):
        while 1:
            try:
                name = self.locals.__pages__.pop()
            except IndexError:
                raise ApplicationError('pop_page: page stack is empty')
            self.__leave_page(name)
            self.locals.__page__ = name
            self.app.load_page(self)
            if target_page is None or target_page == name:
                break

    def set_request(self, req):
        self.request = req

    def req_equals(self, name):
        try:
            if self.request.field_value(name): 
                return 1
        except KeyError:
            pass
        return 0

    def base_url(self):
        return self.app.base_url()

    def current_url(self):
        uri = self.request.get_uri()
        scheme, netloc, path = urlparse.urlparse(uri)[:3]
        return path

    def absolute_base_url(self):
        this_path = urlparse.urlparse(self.request.get_uri())[2]
        base_path = urlparse.urlparse(self.app.base_url())[2]
        pos = this_path.find(base_path)
        new_base = this_path[:pos + len(base_path)]
        if new_base[-1] != '/':
            new_base = new_base + '/'
        return new_base

    def redirect_url(self, loc):
        new_base = self.absolute_base_url()
        return urlparse.urlunparse(('http', self.request.get_servername(),
                                    new_base + loc, '', '', ''))

    def redirect(self, loc):
        scheme, netloc, path = urlparse.urlparse(loc)[:3]
        if scheme or netloc or (path and path[0] == '/'):
            raise Redirect(loc)
        # Redirect from base url - turn into full URL redirect
        new_loc = self.redirect_url(loc)
        raise Redirect(new_loc)


# Basic application processing sequence is implemented in a class
# which is then subclassed in cgiapp and modapp.
class Application(ResourceMixin):

    def __init__(self, base_url):
        ResourceMixin.__init__(self)
        self.register_tagclasses(*tags.tags)
        # Use a base_url of '/' if not provided
        if base_url:
            self.__base_url = base_url
        else:
            self.__base_url = '/'

    def run(self, req):
        '''Process a single browser request
        '''
        ctx = None
        try:
            ctx = self.create_context()
            ctx.set_request(req)
            self.load_session(ctx)
            self.load_page(ctx)
            if self.validate_request(ctx):
                self.merge_request(ctx)
                self.process_request(ctx)
            self.display_response(ctx)
            self.save_session(ctx)
            ctx.flush_content()
        except Redirect, e:
            self.save_session(ctx)
            return ctx.send_redirect(e.loc)
        except:
            self.handle_exception(ctx, req)
        return req.status()

    def format_exception(self):
        import traceback
        import linecache
        etype, value, tb = sys.exc_info()
        list = traceback.format_exception(etype, value, tb)
        pyexc = ''.join(list)
        list = []
        list.append('Template traceback (most recent call last):')
        for tup in self.template_traceback(tb):
            list.append('  File "%s", line %s, in %s' % tup)
            line = linecache.getline(tup[0], tup[1]).rstrip()
            if line:
                list.append(line)
        htmlexc = '\n'.join(list)
        return pyexc, htmlexc

    def handle_exception(self, ctx, req):
        pyexc, htmlexc = map(tags.escape, self.format_exception())
        req.write_header('Content-Type', 'text/html')
        req.end_headers()
        try:
            tmp_ctx = self.create_context()
            tmp_ctx.set_request(req)
            tmp_ctx.locals.python_exc = pyexc
            tmp_ctx.locals.html_exc = htmlexc
            templ = self.load_template('traceback.html')
            templ.to_html(tmp_ctx)
            tmp_ctx.flush_content()
        except:
            req.write_content('<pre>')
            req.write_content(htmlexc)
            req.write_content('\n\n')
            req.write_content(pyexc)
            req.write_content('</pre>')
        if ctx:
            self.remove_session(ctx)

    def template_traceback(self, tb):
        list = []
        prev = None
        while tb is not None:
            f = tb.tb_frame
            if f.f_locals.has_key('self'):
                obj = f.f_locals['self']
                if hasattr(obj, 'line_num'):
                    tag = (obj.filename, obj.line_num, obj.name)
                    if tag != prev:
                        list.append(tag)
                        prev = tag
            tb = tb.tb_next
        return list

    def load_session(self, ctx):
        ctx.load_session()

    def save_session(self, ctx):
        ctx.save_session()

    def remove_session(self, ctx):
        ctx.remove_session()

    def validate_request(self, ctx):
        return 1

    def base_url(self):
        return self.__base_url

    def merge_request(self, ctx):
        ctx.merge_request()

    def pickle_sign(self, text):
        return ''

    def pickle_unsign(self, text):
        return ''

# ------------------------------------------------------------------
# Page processing handlers
# ------------------------------------------------------------------


# A caching page module loader which only reloads the page module it
# has been modified.
class PageModuleMixin:

    '''Caching module loader
    '''

    def __init__(self, base_dir, start_page):
        self.__base_dir = base_dir
        self.__start_page = start_page
        self.__cache = {}

    def module_path(self):
        return self.__base_dir

    def start_page(self):
        return self.__start_page

    def load_page(self, ctx):
        if ctx.locals.__page__:
            name = ctx.locals.__page__
        else:
            name = self.__start_page
            ctx.set_page(name)
        self.load_page_module(ctx, name)

    def load_page_module(self, ctx, name):
        if self.__base_dir == '.':
            path = name
        else:
            path = os.path.join(self.__base_dir, name)
        page = self.__cache.get(path)
        if page:
            mtime = os.stat(page.__file__)[stat.ST_MTIME]
            if mtime > page.__mtime__:
                reload(page)
                page.__mtime__ = mtime
        if not page:
            dirname, name = os.path.split(path)
            file, filepath, desc = imp.find_module(name, [dirname])
            page = sys.modules.get(name)
            if not page or not page.__file__.startswith(filepath):
                try:
                    page = imp.load_module(name, file, filepath, desc)
                finally:
                    if file:
                        file.close()
            if not (hasattr(page, 'page_enter') or
                    hasattr(page, 'page_leave') or
                    hasattr(page, 'page_process') or
                    hasattr(page, 'page_display')):
                raise ApplicationError('module "%s" does not define one of page_enter, page_leave, page_process or page_display' % (name))
            page.__mtime__ = os.stat(page.__file__)[stat.ST_MTIME]
            self.__cache[path] = page
        ctx.page = page

    def page_enter(self, ctx, args):
        if hasattr(ctx.page, 'page_enter'):
            func = getattr(ctx.page, 'page_enter')
            func(ctx, *args)

    def page_leave(self, ctx):
        if hasattr(ctx.page, 'page_leave'):
            func = getattr(ctx.page, 'page_leave')
            func(ctx)

    def process_request(self, ctx):
        if hasattr(ctx.page, 'page_process'):
            func = getattr(ctx.page, 'page_process')
            func(ctx)

    def display_response(self, ctx):
        if hasattr(ctx.page, 'page_display'):
            func = getattr(ctx.page, 'page_display')
            func(ctx)


# A page loader in which pages are implemented in objects.
class PageObjectMixin:

    def __init__(self, start_page):
        self.__start_page = start_page
        self.__page_objects = {}

    def module_path(self):
        pass

    def start_page(self):
        return self.__start_page

    def register_page(self, name, obj):
        self.__page_objects[name] = obj

    def load_page(self, ctx):
        if ctx.locals.__page__:
            name = ctx.locals.__page__
        else:
            name = self.__start_page
            ctx.set_page(name)
        ctx.page = name

    def page_enter(self, ctx, args):
        obj = self.__page_objects[ctx.page]
        if hasattr(obj, 'page_enter'):
            func = getattr(obj, 'page_enter')
            func(ctx, *args)

    def page_leave(self, ctx):
        obj = self.__page_objects[ctx.page]
        if hasattr(obj, 'page_leave'):
            func = getattr(obj, 'page_leave')
            func(ctx)

    def process_request(self, ctx):
        obj = self.__page_objects[ctx.page]
        if hasattr(obj, 'page_process'):
            func = getattr(obj, 'page_process')
            func(ctx)

    def display_response(self, ctx):
        obj = self.__page_objects[ctx.page]
        if hasattr(obj, 'page_display'):
            func = getattr(obj, 'page_display')
            func(ctx)

# ------------------------------------------------------------------
# Prepackaged application contexts
# ------------------------------------------------------------------


# Simple application context simply stores field names in form to
# allow empty fields to be set to None.  Also saves state in hidden
# form fields.
class SimpleAppContext(AppContext,
                       NameRecorderMixin,
                       HiddenFieldSessionMixin):

    def __init__(self, app):
        AppContext.__init__(self, app)
        NameRecorderMixin.__init__(self)
        HiddenFieldSessionMixin.__init__(self)

    def form_close(self):
        use_multipart_enc = NameRecorderMixin.form_close(self)
        HiddenFieldSessionMixin.form_close(self)
        return use_multipart_enc


# Session state is stored server side
class SessionAppContext(AppContext,
                        NameRecorderMixin,
                        SessionServerContextMixin):

    def __init__(self, app):
        AppContext.__init__(self, app)
        NameRecorderMixin.__init__(self)
        SessionServerContextMixin.__init__(self)

# ------------------------------------------------------------------
# Prepackaged application objects
# ------------------------------------------------------------------


# Page processing is implemented in methods of page objects
class SimpleApp(PickleSignMixin,
                Application,
                CachingTemplateLoaderMixin,
                PageObjectMixin):

    def __init__(self, base_url, template_path, start_page, secret):
        Application.__init__(self, base_url)
        PickleSignMixin.__init__(self, secret)
        CachingTemplateLoaderMixin.__init__(self, template_path)
        PageObjectMixin.__init__(self, start_page)

    def create_context(self):
        return SimpleAppContext(self)


# Same as SimpleApp with server-side session support
class SimpleSessionApp(PickleSignMixin,
                       Application,
                       CachingTemplateLoaderMixin,
                       PageObjectMixin,
                       SessionServerAppMixin):

    def __init__(self, base_url, template_path, start_page, secret,
                 session_appid, session_server = 'localhost', server_port = 34343, session_age = 1800):
        Application.__init__(self, base_url)
        PickleSignMixin.__init__(self, secret)
        CachingTemplateLoaderMixin.__init__(self, template_path)
        PageObjectMixin.__init__(self, start_page)
        SessionServerAppMixin.__init__(self, session_appid, session_server, server_port, session_age)

    def create_context(self):
        return SessionAppContext(self)


# Page processing is implemented in functions of page modules
class ModularApp(PickleSignMixin,
                 Application,
                 CachingTemplateLoaderMixin,
                 PageModuleMixin):

    def __init__(self, base_url, module_path, template_path, start_page, secret):
        Application.__init__(self, base_url)
        PickleSignMixin.__init__(self, secret)
        CachingTemplateLoaderMixin.__init__(self, template_path)
        PageModuleMixin.__init__(self, module_path, start_page)

    def create_context(self):
        return SimpleAppContext(self)


# Same as ModularApp with server-side session support
class ModularSessionApp(PickleSignMixin,
                        Application,
                        CachingTemplateLoaderMixin,
                        PageModuleMixin,
                        SessionServerAppMixin):

    def __init__(self, base_url, module_path, template_path, start_page, secret,
                 session_appid, session_server = 'localhost', server_port = 34343, session_age = 1800):
        Application.__init__(self, base_url)
        PickleSignMixin.__init__(self, secret)
        CachingTemplateLoaderMixin.__init__(self, template_path)
        PageModuleMixin.__init__(self, module_path, start_page)
        SessionServerAppMixin.__init__(self, session_appid, session_server, server_port, session_age)

    def create_context(self):
        return SessionAppContext(self)



#!/usr/bin/python
#
#       Test fragments of example code to ensure they behave as
#       advertised.
#
#       Functionally, this is similar to doctest, however our input
#       is more regular than that seen by doctest, and therefore our
#       parsing is more relaxed.
#

import sys, os, traceback, types

PS1 = '>>>'
PS2 = '...'

class DummyFile:
    """
    A simple file-like class that captures output writen to it and
    regurgitates it on demand.
    """

    def __init__(self):
        self.buf = []

    def write(self, s):
        self.buf.append(s)
    
    def flush(self):
        pass

    def get(self):
        result = "".join(self.buf)
        self.buf = []
        return result


class ExampleFailure(Exception):
    pass


class ExampleTest:
    def __init__(self, filename):
        self.filename = filename
        self.result = ""
        self.example = []

    def report(self):
        if self.result:
            print "=" * 70
            print "Testing \"%s\" failed," % self.filename,
            print self.result
            print

    def test(self):
        try:
            if not self.example:
                self.load_and_parse()
            self.result = ""
            self.execute_verify_output()
        except ExampleFailure, msg:
            self.result = msg

        return self.result == ""


    def load_and_parse(self):
        try:
            lines = open(self.filename).readlines()
        except IOError, (eno, estr):
            raise ExampleFailure("could not load: %s" % estr)

        src, want = "", ""
        cmd_lineno = 0
        cur_lineno = 0
        while lines:
            line = lines.pop(0)
            cur_lineno += 1

            if line.startswith(PS1):
                if src:
                    self.example.append((src, cmd_lineno, want))
                cmd_lineno = cur_lineno
                src = line[4:]
                want = ""
            elif line.startswith(PS2):
                if not src:
                    raise ExampleFailure("Line %d: %s line with no preceeding %s line" % (cur_lineno, PS2, PS1))
                src += line[4:]
            else:
                if not src:
                    raise ExampleFailure("Line %s: output with no preceeding %s line" % (cur_lineno, PS1))
                want += line
        if src:
            self.example.append((src, cmd_lineno, want))


    def execute_verify_output(self):
        saved_stdout = sys.stdout
        sys.stdout = dummyfile = DummyFile()
        namespace = {}
        for src, lineno, want in self.example:
            got = None
            try:
                code = compile(src, '<string>', 'single')
                exec code in namespace
                got = dummyfile.get()
            except:
                # Is it a wanted exception?
                if want.find("Traceback (innermost last):\n") == 0 or \
                   want.find("Traceback (most recent call last):\n") == 0:
                    # Only compare exception type and value - the rest of
                    # the traceback isn't necessary.
                    exc_type, exc_val = sys.exc_info()[:2]
                    got = traceback.format_exception_only(exc_type, exc_val)[-1]
                    want = want.split('\n')[-2].strip() + '\n'
                else:
                    exc_type, exc_val, exc_tb = sys.exc_info()
                    if type(exc_type) == types.ClassType:
                        exc_type = exc_type.__name__
                    exc_tb = exc_tb.tb_next
                    psrc = PS1 + " " + src.rstrip().replace("\n", PS2 + " ")
                    pexp = "".join(traceback.format_exception(exc_type, exc_val, exc_tb)).rstrip()

                    self.result = "Line %d:\n%s\n%s" % (lineno, psrc, pexp)
                    del exc_tb
                    break

            if got != want:
                self.result = "Line %d: output does not match example\nExpected:\n%s\nGot:\n%s" % (lineno, want.rstrip(), got.rstrip())
                break

        sys.stdout = saved_stdout


def files_from_directory(dir):
    filenames = []
    for fn in os.listdir(dir):
        filename = os.path.join(dir, fn)
        if fn[0] != '.' and os.path.isfile(filename) \
               and not filename.endswith('.py'):
            filenames.append(filename)
    filenames.sort()
    return filenames


def main():
    if len(sys.argv) > 1:
        filenames = sys.argv[1:]
    else:
        filenames = files_from_directory("doctest")

    fail_cnt = total_cnt = 0
    failures = []

    for filename in filenames:
        if os.path.isfile(filename):
            total_cnt += 1
            e = ExampleTest(filename)
            if not e.test():
                failures.append(e)
                fail_cnt += 1
                sys.stdout.write("!")
            else:
                sys.stdout.write(".")
            sys.stdout.flush()
    sys.stdout.write("\n")

    for e in failures:
        e.report()

    print "%d of %d tests failed" % (fail_cnt, total_cnt)

    sys.exit(fail_cnt > 0)


if __name__ == '__main__':
    main()




#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
import cgi
import sys
import os


class FileField:

    def __init__(self, field):
        self.filename = field.filename
        self.file = field.file
        self.type = field.type

    def __repr__(self):
        return 'FileField(%s, %s, %s)' % (self.filename, self.file, self.type)


class RequestFields:

    def __init__(self, fields):
        self.__fields = fields

    def has_field(self, name):
        return self.__fields.has_key(name)

    def field_value(self, name):
        field = self.__fields[name]
        if isinstance(field, type([])):
            if isinstance(field[0], type('')):
                # For checkboxes, this is a vec of strings in mod_python 2
                return field
            # But a vec of utils.FieldStorage objects in mod_python 3
            return [f.value for f in field]
        elif isinstance(field, type('')):
            return field                # Weird - blame mod_apache?
        else:
            return field.value

    def field_file(self, name):
        field = self.__fields[name]
        if isinstance(field, type([])):
            return [FileField(f) for f in field]
        if field.type.startswith('multipart'):
            return [FileField(f) for f in field.value]
        else:
            return [FileField(field)]

    def field_names(self):
        return self.__fields.keys()


class Request(RequestFields):

    def __init__(self, fields = None):
        if fields is None:
            fields = cgi.FieldStorage()
        RequestFields.__init__(self, fields)
        self.__status = 200

    def get_uri(self):
        return os.environ.get('REQUEST_URI')

    def get_path_info(self):
        return self.__fcgi.env.get('PATH_INFO')

    def get_servername(self):
        return os.environ.get('HTTP_HOST')

    def get_header(self, name):
        env_name = 'HTTP_' + name.upper().replace('-', '_')
        return os.environ.get(env_name)

    def write_header(self, name, value):
        sys.stdout.write('%s: %s\n' % (name, value))

    def end_headers(self):
        sys.stdout.write('\n')

    def redirect(self, loc):
        self.write_header('Location', loc)
        self.write_header('Status', '301 Moved Permanently')
        self.end_headers()

    def write_content(self, data):
        sys.stdout.write(data)
        sys.stdout.flush()

    def set_status(self, status):
        self.__status = status

    def status(self):
        return self.__status



#!/usr/bin/python
from albatross import RandomModularSessionApp
from albatross.cgiapp import Request


app = RandomModularSessionApp(base_url='randompage.py',
                              page_path='pages',
                              start_page='tree',
                              secret='-=-secret-=-',
                              session_appid='random')


if __name__ == '__main__':
    app.run(Request())



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
#
# Dummy Request objects
#
# $Id: request.py,v 1.6 2003/09/21 04:40:31 djc Exp $
#

import string, re

class DummyCookieRequest:
    """
    A dummy Request-like class that serves to record the user's session id
    """
    def __init__(self):
        self.cookie = None

    def write_header(self, hdr, value):
        if hdr == 'Set-Cookie':
            self.cookie = value

    def get_header(self, hdr):
        if hdr == 'Cookie':
            return self.cookie

    def end_headers(self):
        pass

class DummyFieldRequest:
    def __init__(self):
        self.fieldre = re.compile(r'<input type="hidden" name="(__al[a-zA-Z0-9]*__)" value="([^"]*)" />', re.I|re.M|re.S)
        self.fields = {}
        self.__content_parts = []

    def has_field(self, field):
        return self.fields.has_key(field)

    def field_value(self, field):
        return self.fields[field]

    def write_content(self, data):
        self.__content_parts.append(data)

    def flush_content(self):
        data = string.join(self.__content_parts, '')
        match = self.fieldre.search(data)
        if match:
            key = match.group(1)
            value = match.group(2)
            self.fields[key] = value





from albatross import ModularApp
from albatross.apacheapp import Request


app = ModularApp(base_url='mpperf',
                 module_path='-=-install_dir-=-',
                 template_path='-=-install_dir-=-',
                 start_page='main',
                 secret='-=-secret-=-')


def handler(req):
    return app.run(Request(req))



from albatross import SimpleContext


class Node:

    def __init__(self, name, children = None):
        self.name = name
        if children is not None:
            self.children = children


ctx = SimpleContext('.')
ctx.locals.tree = Node('a', [Node('b', [Node('c'),
                                        Node('d')]),
                             Node('e', [Node('f', [Node('g', [Node('h'),
                                                              Node('i')])]),
                                        Node('j'),
                                        Node('k', [Node('l'),
                                                   Node('m')])])])
templ = ctx.load_template('tree.html')
templ.to_html(ctx)
ctx.flush_content()



import sys
sys.path.insert(0, '../..')
import install
from install import Installer

i = Installer('alsamp/form2', cgi = 1)
i.add_exec('form.py')
i.add_html('form.html')
i.add_html('form-display.html')
i.install()



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# 
# Test whitespace.
#
# $Id: whitespace.py,v 1.8 2003/01/24 11:34:31 andrewm Exp $

import unittest
import albatross
import albatross_test

class WhitespaceCase(albatross_test.AlbatrossTestCase):
    template_dir = "whitespace"

    def check_default_ws(self):
        self.html_test("default-ws.html", "123")

    def check_strip(self):
        self.html_test("strip.html", "123")

    def check_indent(self):
        self.html_test("indent.html", "1  2  3")

    def check_newline(self):
        self.html_test("newline.html", "1\n2\n3")

    def check_all(self):
        self.html_test("all.html", "1\n       2\n    \t3")

    def check_bogus(self):
        self.assertRaises(albatross.ApplicationError, self.html_test, "bogus.html", "(none)")

class WhitespaceSuite(unittest.TestSuite):
    test_list = (
        "check_default_ws",
        "check_strip",
        "check_indent",
        "check_newline",
        "check_all",
        "check_bogus",
    )
    def __init__(self):
        unittest.TestSuite.__init__(self, map(WhitespaceCase, self.test_list))

def suite():
    return WhitespaceSuite()

if __name__ == '__main__':
    unittest.main(defaultTest='suite')



node_num = 0


class Node:

    def __init__(self, name, children = None):
        self.name = name
        if children is not None:
            self.children = children
        self.selected = 0
        global node_num
        self.node_num = node_num
        node_num = node_num + 1

    def albatross_alias(self):
        return 'node%d' % self.node_num



#
# Copyright 2001 by Object Craft P/L, Melbourne, Australia.
#
# LICENCE - see LICENCE file distributed with this software for details.
#
# Stuff shared by all albatross modules

# Base exception class used by all Albatross exceptions
class AlbatrossError(Exception):
    pass


# Generic causes of exceptions:
class UserError(AlbatrossError):
    pass


class ApplicationError(AlbatrossError):
    pass


class InternalError(AlbatrossError):
    pass


class ServerError(AlbatrossError):
    pass


# Specific exceptions:
class SecurityError(UserError):
    pass


class TemplateLoadError(ApplicationError):
    pass



#!/usr/bin/python

print 'Content-Type: text/html'
print
print '<html>'
print ' <head>'
print '  <title>My CGI application</title>'
print ' </head>'
print ' <body>'
print '  Hello from my simple CGI application!'
print ' </body>'
print '</html>'
